# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: partitions_service.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import partitions_common_pb2 as partitions__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18partitions_service.proto\x12\x1e\x61rmonik.api.grpc.v1.partitions\x1a\x17partitions_common.proto2\x8d\x02\n\nPartitions\x12\x81\x01\n\x0eListPartitions\x12\x35.armonik.api.grpc.v1.partitions.ListPartitionsRequest\x1a\x36.armonik.api.grpc.v1.partitions.ListPartitionsResponse\"\x00\x12{\n\x0cGetPartition\x12\x33.armonik.api.grpc.v1.partitions.GetPartitionRequest\x1a\x34.armonik.api.grpc.v1.partitions.GetPartitionResponse\"\x00\x42!\xaa\x02\x1e\x41rmoniK.Api.gRPC.V1.Partitionsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'partitions_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\036ArmoniK.Api.gRPC.V1.Partitions'
  _globals['_PARTITIONS']._serialized_start=86
  _globals['_PARTITIONS']._serialized_end=355
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: auth_service.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import auth_common_pb2 as auth__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x61uth_service.proto\x12\x18\x61rmonik.api.grpc.v1.auth\x1a\x11\x61uth_common.proto2\x87\x01\n\x0e\x41uthentication\x12u\n\x0eGetCurrentUser\x12/.armonik.api.grpc.v1.auth.GetCurrentUserRequest\x1a\x30.armonik.api.grpc.v1.auth.GetCurrentUserResponse\"\x00\x42\x1b\xaa\x02\x18\x41rmoniK.Api.gRPC.V1.Authb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'auth_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\030ArmoniK.Api.gRPC.V1.Auth'
  _globals['_AUTHENTICATION']._serialized_start=68
  _globals['_AUTHENTICATION']._serialized_end=203
# @@protoc_insertion_point(module_scope)



from .partitions import ArmoniKPartitions, PartitionFieldFilter
from .sessions import ArmoniKSessions, SessionFieldFilter
from .submitter import ArmoniKSubmitter
from .tasks import ArmoniKTasks, TaskFieldFilter
from .results import ArmoniKResults, ResultFieldFilter
from .versions import ArmoniKVersions
from .events import ArmoniKEvents
from .health_checks import ArmoniKHealthChecks

__all__ = [
    "ArmoniKPartitions",
    "ArmoniKSessions",
    "ArmoniKSubmitter",
    "ArmoniKTasks",
    "ArmoniKResults",
    "ArmoniKVersions",
    "ArmoniKEvents",
    "ArmoniKHealthChecks",
    "TaskFieldFilter",
    "PartitionFieldFilter",
    "SessionFieldFilter",
    "ResultFieldFilter",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: events_service.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import events_common_pb2 as events__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x65vents_service.proto\x12\x1a\x61rmonik.api.grpc.v1.events\x1a\x13\x65vents_common.proto2\x84\x01\n\x06\x45vents\x12z\n\tGetEvents\x12\x34.armonik.api.grpc.v1.events.EventSubscriptionRequest\x1a\x35.armonik.api.grpc.v1.events.EventSubscriptionResponse0\x01\x42\x1d\xaa\x02\x1a\x41rmoniK.Api.gRPC.V1.Eventsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'events_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\032ArmoniK.Api.gRPC.V1.Events'
  _globals['_EVENTS']._serialized_start=74
  _globals['_EVENTS']._serialized_end=206
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

import pytest

from dataclasses import dataclass
from armonik.common.filter import Filter, StringFilter, BooleanFilter, NumberFilter
from armonik.protogen.common.filters_common_pb2 import FilterBoolean
from google.protobuf.message import Message


@dataclass
class DummyMessage(Message):
    pass


@dataclass
class DummyMessageAnd(Message):
    pass


@dataclass
class Field(Message):
    pass


@pytest.mark.parametrize(
    "filt,inverted",
    [
        (
            StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) == "Test",
            StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) != "Test",
        ),
        (
            StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage).contains("Test"),
            ~(StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage).contains("Test")),
        ),
        (
            BooleanFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage),
            BooleanFilter(
                Field(),
                DummyMessage,
                DummyMessageAnd,
                DummyMessage,
                FilterBoolean,
                None,
                False,
            ),
        ),
        (
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) > 0,
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) <= 0,
        ),
        (
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) >= 0,
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) < 0,
        ),
        (
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) < 0,
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) >= 0,
        ),
        (
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) <= 0,
            NumberFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage) > 0,
        ),
    ],
)
def test_inversion(filt: Filter, inverted: Filter):
    assert filt.operator != inverted.operator or filt.value == (
        not inverted.value
    )  # In case of BooleanFilter, the value is inverted, not the operator
    assert (~filt).operator == inverted.operator and (~filt).value == inverted.value
    assert filt.operator == (~(~filt)).operator and filt.value == (~(~filt)).value


@pytest.mark.parametrize(
    "filt",
    [
        (StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage).startswith("Test")),
        (StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage).endswith("Test")),
        (StringFilter(Field(), DummyMessage, DummyMessageAnd, DummyMessage)),  # No op
    ],
)
def test_inversion_raises(filt: Filter):
    with pytest.raises(Exception):
        test = ~filt
        print(test)



from typing import Callable, cast, List

from grpc import Channel

from .results import ArmoniKResults
from ..common import (
    EventTypes,
    Filter,
    NewTaskEvent,
    NewResultEvent,
    ResultOwnerUpdateEvent,
    ResultStatusUpdateEvent,
    TaskStatusUpdateEvent,
    ResultStatus,
    Event,
)
from .results import ResultFieldFilter
from ..protogen.client.events_service_pb2_grpc import EventsStub
from ..protogen.common.events_common_pb2 import EventSubscriptionRequest
from ..protogen.common.results_filters_pb2 import Filters as rawResultFilters
from ..protogen.common.tasks_filters_pb2 import Filters as rawTaskFilters


class ArmoniKEvents:
    _events_obj_mapping = {
        "new_result": NewResultEvent,
        "new_task": NewTaskEvent,
        "result_owner_update": ResultOwnerUpdateEvent,
        "result_status_update": ResultStatusUpdateEvent,
        "task_status_update": TaskStatusUpdateEvent,
    }

    def __init__(self, grpc_channel: Channel):
        """Events service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = EventsStub(grpc_channel)
        self._results_client = ArmoniKResults(grpc_channel)

    def get_events(
        self,
        session_id: str,
        event_types: List[EventTypes],
        event_handlers: List[Callable[[str, EventTypes, Event], bool]],
        task_filter: Filter | None = None,
        result_filter: Filter | None = None,
    ) -> None:
        """Get events that represents updates of result and tasks data.

        Args:
            session_id: The ID of the session.
            event_types: The list of the types of event to catch.
            event_handlers: The list of handlers that process the events. Handlers are evaluated in he order they are provided.
                An handler takes three positional arguments: the ID of the session, the type of event and the event as an object.
                An handler returns a boolean, if True the process continues, otherwise the stream is closed and the service stops
                listening to new events.
            task_filter: A filter on tasks.
            result_filter: A filter on results.

        """
        request = EventSubscriptionRequest(session_id=session_id, returned_events=event_types)
        if task_filter:
            request.tasks_filters = (
                cast(rawTaskFilters, task_filter.to_disjunction().to_message()),
            )
        if result_filter:
            request.results_filters = (
                cast(rawResultFilters, result_filter.to_disjunction().to_message()),
            )

        streaming_call = self._client.GetEvents(request)
        for message in streaming_call:
            event_type = message.WhichOneof("update")
            if any(
                [
                    event_handler(
                        session_id,
                        EventTypes.from_string(event_type),
                        self._events_obj_mapping[event_type].from_raw_event(
                            getattr(message, event_type)
                        ),
                    )
                    for event_handler in event_handlers
                ]
            ):
                break

    def wait_for_result_availability(self, result_id: str, session_id: str) -> None:
        """Wait until a result is ready i.e its status updates to COMPLETED.

        Args:
            result_id: The ID of the result.
            session_id: The ID of the session.

        Raises:
            RuntimeError: If the result status is ABORTED.
        """

        def handler(session_id, event_type, event):
            if not isinstance(event, ResultStatusUpdateEvent):
                raise ValueError("Handler should receive event of type 'ResultStatusUpdateEvent'.")
            if event.status == ResultStatus.COMPLETED:
                return False
            elif event.status == ResultStatus.ABORTED:
                raise RuntimeError(f"Result {result.name} with ID {result_id} is aborted.")
            return True

        result = self._results_client.get_result(result_id)
        if result.status == ResultStatus.COMPLETED:
            return
        elif result.status == ResultStatus.ABORTED:
            raise RuntimeError(f"Result {result.name} with ID {result_id} is aborted.")

        self.get_events(
            session_id,
            [EventTypes.RESULT_STATUS_UPDATE],
            [handler],
            result_filter=(ResultFieldFilter.RESULT_ID == result_id),
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sessions_common.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import objects_pb2 as objects__pb2
from . import session_status_pb2 as session__status__pb2
from . import sessions_fields_pb2 as sessions__fields__pb2
from . import sessions_filters_pb2 as sessions__filters__pb2
from . import sort_direction_pb2 as sort__direction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15sessions_common.proto\x12\x1c\x61rmonik.api.grpc.v1.sessions\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\robjects.proto\x1a\x14session_status.proto\x1a\x15sessions_fields.proto\x1a\x16sessions_filters.proto\x1a\x14sort_direction.proto\"\xd1\x03\n\nSessionRaw\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x41\n\x06status\x18\x02 \x01(\x0e\x32\x31.armonik.api.grpc.v1.session_status.SessionStatus\x12\x19\n\x11\x63lient_submission\x18\x08 \x01(\x08\x12\x19\n\x11worker_submission\x18\t \x01(\x08\x12\x15\n\rpartition_ids\x18\x03 \x03(\t\x12\x31\n\x07options\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x63\x61ncelled_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tpurged_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\ndeleted_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x08\x64uration\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xd9\x02\n\x13ListSessionsRequest\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x36\n\x07\x66ilters\x18\x03 \x01(\x0b\x32%.armonik.api.grpc.v1.sessions.Filters\x12\x44\n\x04sort\x18\x04 \x01(\x0b\x32\x36.armonik.api.grpc.v1.sessions.ListSessionsRequest.Sort\x12\x19\n\x11with_task_options\x18\x05 \x01(\x08\x1a\x87\x01\n\x04Sort\x12\x39\n\x05\x66ield\x18\x01 \x01(\x0b\x32*.armonik.api.grpc.v1.sessions.SessionField\x12\x44\n\tdirection\x18\x02 \x01(\x0e\x32\x31.armonik.api.grpc.v1.sort_direction.SortDirection\"\x82\x01\n\x14ListSessionsResponse\x12:\n\x08sessions\x18\x01 \x03(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\x12\x0c\n\x04page\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\"\'\n\x11GetSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"O\n\x12GetSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\"*\n\x14\x43\x61ncelSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"R\n\x15\x43\x61ncelSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\"l\n\x14\x43reateSessionRequest\x12=\n\x13\x64\x65\x66\x61ult_task_option\x18\x01 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12\x15\n\rpartition_ids\x18\x02 \x03(\t\"(\n\x12\x43reateSessionReply\x12\x12\n\nsession_id\x18\x01 \x01(\t\")\n\x13PauseSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"Q\n\x14PauseSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\"*\n\x14ResumeSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"R\n\x15ResumeSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\")\n\x13PurgeSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"Q\n\x14PurgeSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\"*\n\x14\x44\x65leteSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"R\n\x15\x44\x65leteSessionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRaw\"K\n\x15StopSubmissionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0e\n\x06\x63lient\x18\x02 \x01(\x08\x12\x0e\n\x06worker\x18\x03 \x01(\x08\"S\n\x16StopSubmissionResponse\x12\x39\n\x07session\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.sessions.SessionRawB\x1f\xaa\x02\x1c\x41rmoniK.Api.gRPC.V1.Sessionsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sessions_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\034ArmoniK.Api.gRPC.V1.Sessions'
  _globals['_SESSIONRAW']._serialized_start=227
  _globals['_SESSIONRAW']._serialized_end=692
  _globals['_LISTSESSIONSREQUEST']._serialized_start=695
  _globals['_LISTSESSIONSREQUEST']._serialized_end=1040
  _globals['_LISTSESSIONSREQUEST_SORT']._serialized_start=905
  _globals['_LISTSESSIONSREQUEST_SORT']._serialized_end=1040
  _globals['_LISTSESSIONSRESPONSE']._serialized_start=1043
  _globals['_LISTSESSIONSRESPONSE']._serialized_end=1173
  _globals['_GETSESSIONREQUEST']._serialized_start=1175
  _globals['_GETSESSIONREQUEST']._serialized_end=1214
  _globals['_GETSESSIONRESPONSE']._serialized_start=1216
  _globals['_GETSESSIONRESPONSE']._serialized_end=1295
  _globals['_CANCELSESSIONREQUEST']._serialized_start=1297
  _globals['_CANCELSESSIONREQUEST']._serialized_end=1339
  _globals['_CANCELSESSIONRESPONSE']._serialized_start=1341
  _globals['_CANCELSESSIONRESPONSE']._serialized_end=1423
  _globals['_CREATESESSIONREQUEST']._serialized_start=1425
  _globals['_CREATESESSIONREQUEST']._serialized_end=1533
  _globals['_CREATESESSIONREPLY']._serialized_start=1535
  _globals['_CREATESESSIONREPLY']._serialized_end=1575
  _globals['_PAUSESESSIONREQUEST']._serialized_start=1577
  _globals['_PAUSESESSIONREQUEST']._serialized_end=1618
  _globals['_PAUSESESSIONRESPONSE']._serialized_start=1620
  _globals['_PAUSESESSIONRESPONSE']._serialized_end=1701
  _globals['_RESUMESESSIONREQUEST']._serialized_start=1703
  _globals['_RESUMESESSIONREQUEST']._serialized_end=1745
  _globals['_RESUMESESSIONRESPONSE']._serialized_start=1747
  _globals['_RESUMESESSIONRESPONSE']._serialized_end=1829
  _globals['_PURGESESSIONREQUEST']._serialized_start=1831
  _globals['_PURGESESSIONREQUEST']._serialized_end=1872
  _globals['_PURGESESSIONRESPONSE']._serialized_start=1874
  _globals['_PURGESESSIONRESPONSE']._serialized_end=1955
  _globals['_DELETESESSIONREQUEST']._serialized_start=1957
  _globals['_DELETESESSIONREQUEST']._serialized_end=1999
  _globals['_DELETESESSIONRESPONSE']._serialized_start=2001
  _globals['_DELETESESSIONRESPONSE']._serialized_end=2083
  _globals['_STOPSUBMISSIONREQUEST']._serialized_start=2085
  _globals['_STOPSUBMISSIONREQUEST']._serialized_end=2160
  _globals['_STOPSUBMISSIONRESPONSE']._serialized_start=2162
  _globals['_STOPSUBMISSIONRESPONSE']._serialized_end=2245
# @@protoc_insertion_point(module_scope)



from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKPartitions, PartitionFieldFilter
from armonik.common import Partition


class TestArmoniKPartitions:
    def test_get_partitions(self):
        partitions_client: ArmoniKPartitions = get_client("Partitions")
        partition = partitions_client.get_partition("partition-id")

        assert rpc_called("Partitions", "GetPartition")
        assert isinstance(partition, Partition)
        assert partition.id == "partition-id"
        assert partition.parent_partition_ids == []
        assert partition.pod_reserved == 1
        assert partition.pod_max == 1
        assert partition.pod_configuration == {}
        assert partition.preemption_percentage == 0
        assert partition.priority == 1

    def test_list_partitions_no_filter(self):
        partitions_client: ArmoniKPartitions = get_client("Partitions")
        num, partitions = partitions_client.list_partitions()

        assert rpc_called("Partitions", "ListPartitions")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert partitions == []

    def test_list_partitions_with_filter(self):
        partitions_client: ArmoniKPartitions = get_client("Partitions")
        num, partitions = partitions_client.list_partitions(PartitionFieldFilter.PRIORITY == 1)

        assert rpc_called("Partitions", "ListPartitions", 2)
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert partitions == []

    def test_service_fully_implemented(self):
        assert all_rpc_called("Partitions")



from . import objects_pb2 as _objects_pb2
from . import result_status_pb2 as _result_status_pb2
from . import session_status_pb2 as _session_status_pb2
from . import task_status_pb2 as _task_status_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SessionList(_message.Message):
    __slots__ = ("sessions",)
    SESSIONS_FIELD_NUMBER: _ClassVar[int]
    sessions: _containers.RepeatedCompositeFieldContainer[_objects_pb2.Session]
    def __init__(self, sessions: _Optional[_Iterable[_Union[_objects_pb2.Session, _Mapping]]] = ...) -> None: ...

class SessionIdList(_message.Message):
    __slots__ = ("session_ids",)
    SESSION_IDS_FIELD_NUMBER: _ClassVar[int]
    session_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, session_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateSessionRequest(_message.Message):
    __slots__ = ("default_task_option", "partition_ids")
    DEFAULT_TASK_OPTION_FIELD_NUMBER: _ClassVar[int]
    PARTITION_IDS_FIELD_NUMBER: _ClassVar[int]
    default_task_option: _objects_pb2.TaskOptions
    partition_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, default_task_option: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., partition_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateSessionReply(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class CreateSmallTaskRequest(_message.Message):
    __slots__ = ("session_id", "task_options", "task_requests")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    TASK_REQUESTS_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    task_options: _objects_pb2.TaskOptions
    task_requests: _containers.RepeatedCompositeFieldContainer[_objects_pb2.TaskRequest]
    def __init__(self, session_id: _Optional[str] = ..., task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., task_requests: _Optional[_Iterable[_Union[_objects_pb2.TaskRequest, _Mapping]]] = ...) -> None: ...

class CreateLargeTaskRequest(_message.Message):
    __slots__ = ("init_request", "init_task", "task_payload")
    class InitRequest(_message.Message):
        __slots__ = ("session_id", "task_options")
        SESSION_ID_FIELD_NUMBER: _ClassVar[int]
        TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
        session_id: str
        task_options: _objects_pb2.TaskOptions
        def __init__(self, session_id: _Optional[str] = ..., task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ...) -> None: ...
    INIT_REQUEST_FIELD_NUMBER: _ClassVar[int]
    INIT_TASK_FIELD_NUMBER: _ClassVar[int]
    TASK_PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    init_request: CreateLargeTaskRequest.InitRequest
    init_task: _objects_pb2.InitTaskRequest
    task_payload: _objects_pb2.DataChunk
    def __init__(self, init_request: _Optional[_Union[CreateLargeTaskRequest.InitRequest, _Mapping]] = ..., init_task: _Optional[_Union[_objects_pb2.InitTaskRequest, _Mapping]] = ..., task_payload: _Optional[_Union[_objects_pb2.DataChunk, _Mapping]] = ...) -> None: ...

class CreateTaskReply(_message.Message):
    __slots__ = ("creation_status_list", "error")
    class TaskInfo(_message.Message):
        __slots__ = ("task_id", "expected_output_keys", "data_dependencies", "payload_id")
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
        DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
        PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
        data_dependencies: _containers.RepeatedScalarFieldContainer[str]
        payload_id: str
        def __init__(self, task_id: _Optional[str] = ..., expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., payload_id: _Optional[str] = ...) -> None: ...
    class CreationStatus(_message.Message):
        __slots__ = ("task_info", "error")
        TASK_INFO_FIELD_NUMBER: _ClassVar[int]
        ERROR_FIELD_NUMBER: _ClassVar[int]
        task_info: CreateTaskReply.TaskInfo
        error: str
        def __init__(self, task_info: _Optional[_Union[CreateTaskReply.TaskInfo, _Mapping]] = ..., error: _Optional[str] = ...) -> None: ...
    class CreationStatusList(_message.Message):
        __slots__ = ("creation_statuses",)
        CREATION_STATUSES_FIELD_NUMBER: _ClassVar[int]
        creation_statuses: _containers.RepeatedCompositeFieldContainer[CreateTaskReply.CreationStatus]
        def __init__(self, creation_statuses: _Optional[_Iterable[_Union[CreateTaskReply.CreationStatus, _Mapping]]] = ...) -> None: ...
    CREATION_STATUS_LIST_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    creation_status_list: CreateTaskReply.CreationStatusList
    error: str
    def __init__(self, creation_status_list: _Optional[_Union[CreateTaskReply.CreationStatusList, _Mapping]] = ..., error: _Optional[str] = ...) -> None: ...

class TaskFilter(_message.Message):
    __slots__ = ("session", "task", "included", "excluded")
    class IdsRequest(_message.Message):
        __slots__ = ("ids",)
        IDS_FIELD_NUMBER: _ClassVar[int]
        ids: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, ids: _Optional[_Iterable[str]] = ...) -> None: ...
    class StatusesRequest(_message.Message):
        __slots__ = ("statuses",)
        STATUSES_FIELD_NUMBER: _ClassVar[int]
        statuses: _containers.RepeatedScalarFieldContainer[_task_status_pb2.TaskStatus]
        def __init__(self, statuses: _Optional[_Iterable[_Union[_task_status_pb2.TaskStatus, str]]] = ...) -> None: ...
    SESSION_FIELD_NUMBER: _ClassVar[int]
    TASK_FIELD_NUMBER: _ClassVar[int]
    INCLUDED_FIELD_NUMBER: _ClassVar[int]
    EXCLUDED_FIELD_NUMBER: _ClassVar[int]
    session: TaskFilter.IdsRequest
    task: TaskFilter.IdsRequest
    included: TaskFilter.StatusesRequest
    excluded: TaskFilter.StatusesRequest
    def __init__(self, session: _Optional[_Union[TaskFilter.IdsRequest, _Mapping]] = ..., task: _Optional[_Union[TaskFilter.IdsRequest, _Mapping]] = ..., included: _Optional[_Union[TaskFilter.StatusesRequest, _Mapping]] = ..., excluded: _Optional[_Union[TaskFilter.StatusesRequest, _Mapping]] = ...) -> None: ...

class SessionFilter(_message.Message):
    __slots__ = ("sessions", "included", "excluded")
    class StatusesRequest(_message.Message):
        __slots__ = ("statuses",)
        STATUSES_FIELD_NUMBER: _ClassVar[int]
        statuses: _containers.RepeatedScalarFieldContainer[_session_status_pb2.SessionStatus]
        def __init__(self, statuses: _Optional[_Iterable[_Union[_session_status_pb2.SessionStatus, str]]] = ...) -> None: ...
    SESSIONS_FIELD_NUMBER: _ClassVar[int]
    INCLUDED_FIELD_NUMBER: _ClassVar[int]
    EXCLUDED_FIELD_NUMBER: _ClassVar[int]
    sessions: _containers.RepeatedScalarFieldContainer[str]
    included: SessionFilter.StatusesRequest
    excluded: SessionFilter.StatusesRequest
    def __init__(self, sessions: _Optional[_Iterable[str]] = ..., included: _Optional[_Union[SessionFilter.StatusesRequest, _Mapping]] = ..., excluded: _Optional[_Union[SessionFilter.StatusesRequest, _Mapping]] = ...) -> None: ...

class GetTaskStatusRequest(_message.Message):
    __slots__ = ("task_ids",)
    TASK_IDS_FIELD_NUMBER: _ClassVar[int]
    task_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, task_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class GetTaskStatusReply(_message.Message):
    __slots__ = ("id_statuses",)
    class IdStatus(_message.Message):
        __slots__ = ("task_id", "status")
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        status: _task_status_pb2.TaskStatus
        def __init__(self, task_id: _Optional[str] = ..., status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ...) -> None: ...
    ID_STATUSES_FIELD_NUMBER: _ClassVar[int]
    id_statuses: _containers.RepeatedCompositeFieldContainer[GetTaskStatusReply.IdStatus]
    def __init__(self, id_statuses: _Optional[_Iterable[_Union[GetTaskStatusReply.IdStatus, _Mapping]]] = ...) -> None: ...

class GetResultStatusRequest(_message.Message):
    __slots__ = ("result_ids", "session_id")
    RESULT_IDS_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    result_ids: _containers.RepeatedScalarFieldContainer[str]
    session_id: str
    def __init__(self, result_ids: _Optional[_Iterable[str]] = ..., session_id: _Optional[str] = ...) -> None: ...

class GetResultStatusReply(_message.Message):
    __slots__ = ("id_statuses",)
    class IdStatus(_message.Message):
        __slots__ = ("result_id", "status")
        RESULT_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        result_id: str
        status: _result_status_pb2.ResultStatus
        def __init__(self, result_id: _Optional[str] = ..., status: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ...) -> None: ...
    ID_STATUSES_FIELD_NUMBER: _ClassVar[int]
    id_statuses: _containers.RepeatedCompositeFieldContainer[GetResultStatusReply.IdStatus]
    def __init__(self, id_statuses: _Optional[_Iterable[_Union[GetResultStatusReply.IdStatus, _Mapping]]] = ...) -> None: ...

class ResultReply(_message.Message):
    __slots__ = ("result", "error", "not_completed_task")
    RESULT_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    NOT_COMPLETED_TASK_FIELD_NUMBER: _ClassVar[int]
    result: _objects_pb2.DataChunk
    error: _objects_pb2.TaskError
    not_completed_task: str
    def __init__(self, result: _Optional[_Union[_objects_pb2.DataChunk, _Mapping]] = ..., error: _Optional[_Union[_objects_pb2.TaskError, _Mapping]] = ..., not_completed_task: _Optional[str] = ...) -> None: ...

class AvailabilityReply(_message.Message):
    __slots__ = ("ok", "error", "not_completed_task")
    OK_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    NOT_COMPLETED_TASK_FIELD_NUMBER: _ClassVar[int]
    ok: _objects_pb2.Empty
    error: _objects_pb2.TaskError
    not_completed_task: str
    def __init__(self, ok: _Optional[_Union[_objects_pb2.Empty, _Mapping]] = ..., error: _Optional[_Union[_objects_pb2.TaskError, _Mapping]] = ..., not_completed_task: _Optional[str] = ...) -> None: ...

class WaitRequest(_message.Message):
    __slots__ = ("filter", "stop_on_first_task_error", "stop_on_first_task_cancellation")
    FILTER_FIELD_NUMBER: _ClassVar[int]
    STOP_ON_FIRST_TASK_ERROR_FIELD_NUMBER: _ClassVar[int]
    STOP_ON_FIRST_TASK_CANCELLATION_FIELD_NUMBER: _ClassVar[int]
    filter: TaskFilter
    stop_on_first_task_error: bool
    stop_on_first_task_cancellation: bool
    def __init__(self, filter: _Optional[_Union[TaskFilter, _Mapping]] = ..., stop_on_first_task_error: bool = ..., stop_on_first_task_cancellation: bool = ...) -> None: ...

class WatchResultRequest(_message.Message):
    __slots__ = ("fetch_statuses", "watch_statuses", "result_ids")
    FETCH_STATUSES_FIELD_NUMBER: _ClassVar[int]
    WATCH_STATUSES_FIELD_NUMBER: _ClassVar[int]
    RESULT_IDS_FIELD_NUMBER: _ClassVar[int]
    fetch_statuses: _containers.RepeatedScalarFieldContainer[_result_status_pb2.ResultStatus]
    watch_statuses: _containers.RepeatedScalarFieldContainer[_result_status_pb2.ResultStatus]
    result_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, fetch_statuses: _Optional[_Iterable[_Union[_result_status_pb2.ResultStatus, str]]] = ..., watch_statuses: _Optional[_Iterable[_Union[_result_status_pb2.ResultStatus, str]]] = ..., result_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class WatchResultStream(_message.Message):
    __slots__ = ("status", "result_ids")
    STATUS_FIELD_NUMBER: _ClassVar[int]
    RESULT_IDS_FIELD_NUMBER: _ClassVar[int]
    status: _result_status_pb2.ResultStatus
    result_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, status: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ..., result_ids: _Optional[_Iterable[str]] = ...) -> None: ...



from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKEvents
from armonik.common import EventTypes, NewResultEvent, ResultStatus


class TestArmoniKEvents:
    def test_get_events_no_filter(self):
        def test_handler(session_id, event_type, event):
            assert session_id == "session-id"
            assert event_type == EventTypes.NEW_RESULT
            assert isinstance(event, NewResultEvent)
            assert event.result_id == "result-id"
            assert event.owner_id == "owner-id"
            assert event.status == ResultStatus.CREATED

        tasks_client: ArmoniKEvents = get_client("Events")
        tasks_client.get_events("session-id", [EventTypes.TASK_STATUS_UPDATE], [test_handler])

        assert rpc_called("Events", "GetEvents")

    def test_service_fully_implemented(self):
        assert all_rpc_called("Events")



import json

from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKVersions


class TestArmoniKVersions:
    def test_list_versions(self):
        """
        Test the list_versions method of ArmoniKVersions client.

        Args:
            grpc_endpoint (str): The gRPC endpoint for the service mock.
            calls_recap_endpoint (str): The endpoint for tracking RPC calls.

        Assertions:
            Ensures that the RPC 'ListVersions' is called on the service 'Versions'.
            Asserts that the 'core' version is returned with correct value.
            Asserts that the 'api' version is returned with correct value.
        """
        versions_client: ArmoniKVersions = get_client("Versions")
        versions = versions_client.list_versions()

        assert rpc_called("Versions", "ListVersions")
        assert versions["core"] == "Unknown"
        with open("../web/package.json", "r") as file:
            assert versions["api"] == json.loads(file.read())["version"] + ".0"

    def test_service_fully_implemented(self):
        """
        Test if all RPCs in the 'Versions' service have been called at least once.

        Args:
            calls_recap_endpoint (str): The endpoint for tracking RPC calls.

        Assertions:
            Ensures that all RPCs in the 'Versions' service have been called at least once.
        """
        assert all_rpc_called("Versions")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tasks_common.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import objects_pb2 as objects__pb2
from . import sort_direction_pb2 as sort__direction__pb2
from . import task_status_pb2 as task__status__pb2
from . import tasks_fields_pb2 as tasks__fields__pb2
from . import tasks_filters_pb2 as tasks__filters__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12tasks_common.proto\x12\x19\x61rmonik.api.grpc.v1.tasks\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\robjects.proto\x1a\x14sort_direction.proto\x1a\x11task_status.proto\x1a\x12tasks_fields.proto\x1a\x13tasks_filters.proto\"\x86\x08\n\x0cTaskDetailed\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x14\n\x0cowner_pod_id\x18\x03 \x01(\t\x12\x17\n\x0finitial_task_id\x18\x16 \x01(\t\x12\x17\n\x0fparent_task_ids\x18\x04 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x05 \x03(\t\x12\x1b\n\x13\x65xpected_output_ids\x18\x06 \x03(\t\x12\x14\n\x0cretry_of_ids\x18\x07 \x03(\t\x12;\n\x06status\x18\x08 \x01(\x0e\x32+.armonik.api.grpc.v1.task_status.TaskStatus\x12\x16\n\x0estatus_message\x18\t \x01(\t\x12\x31\n\x07options\x18\n \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12.\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0csubmitted_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0breceived_at\x18\x12 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0b\x61\x63quired_at\x18\x13 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nstarted_at\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0cprocessed_at\x18\x18 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nded_at\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07pod_ttl\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12;\n\x18\x63reation_to_end_duration\x18\x14 \x01(\x0b\x32\x19.google.protobuf.Duration\x12=\n\x1aprocessing_to_end_duration\x18\x15 \x01(\x0b\x32\x19.google.protobuf.Duration\x12;\n\x18received_to_end_duration\x18\x17 \x01(\x0b\x32\x19.google.protobuf.Duration\x12>\n\x06output\x18\x10 \x01(\x0b\x32..armonik.api.grpc.v1.tasks.TaskDetailed.Output\x12\x14\n\x0cpod_hostname\x18\x11 \x01(\t\x1a(\n\x06Output\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"\xc2\x07\n\x0bTaskSummary\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x14\n\x0cowner_pod_id\x18\n \x01(\t\x12\x17\n\x0finitial_task_id\x18\x16 \x01(\t\x12\x1d\n\x15\x63ount_parent_task_ids\x18\x15 \x01(\x03\x12\x1f\n\x17\x63ount_data_dependencies\x18\x0b \x01(\x03\x12!\n\x19\x63ount_expected_output_ids\x18\r \x01(\x03\x12\x1a\n\x12\x63ount_retry_of_ids\x18\x0e \x01(\x03\x12;\n\x06status\x18\x04 \x01(\x0e\x32+.armonik.api.grpc.v1.task_status.TaskStatus\x12\x16\n\x0estatus_message\x18\t \x01(\t\x12\x31\n\x07options\x18\x03 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0csubmitted_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0breceived_at\x18\x12 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0b\x61\x63quired_at\x18\x13 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nstarted_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0cprocessed_at\x18\x17 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nded_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07pod_ttl\x18\x14 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12;\n\x18\x63reation_to_end_duration\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\x12=\n\x1aprocessing_to_end_duration\x18\x10 \x01(\x0b\x32\x19.google.protobuf.Duration\x12;\n\x18received_to_end_duration\x18\x18 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\r\n\x05\x65rror\x18\x08 \x01(\t\x12\x14\n\x0cpod_hostname\x18\x11 \x01(\t\"\xc1\x02\n\x10ListTasksRequest\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x33\n\x07\x66ilters\x18\x03 \x01(\x0b\x32\".armonik.api.grpc.v1.tasks.Filters\x12>\n\x04sort\x18\x04 \x01(\x0b\x32\x30.armonik.api.grpc.v1.tasks.ListTasksRequest.Sort\x12\x13\n\x0bwith_errors\x18\x05 \x01(\x08\x1a\x81\x01\n\x04Sort\x12\x33\n\x05\x66ield\x18\x01 \x01(\x0b\x32$.armonik.api.grpc.v1.tasks.TaskField\x12\x44\n\tdirection\x18\x02 \x01(\x0e\x32\x31.armonik.api.grpc.v1.sort_direction.SortDirection\"z\n\x11ListTasksResponse\x12\x35\n\x05tasks\x18\x01 \x03(\x0b\x32&.armonik.api.grpc.v1.tasks.TaskSummary\x12\x0c\n\x04page\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\"\x83\x01\n\x19ListTasksDetailedResponse\x12\x36\n\x05tasks\x18\x01 \x03(\x0b\x32\'.armonik.api.grpc.v1.tasks.TaskDetailed\x12\x0c\n\x04page\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\"!\n\x0eGetTaskRequest\x12\x0f\n\x07task_id\x18\x01 \x01(\t\"H\n\x0fGetTaskResponse\x12\x35\n\x04task\x18\x01 \x01(\x0b\x32\'.armonik.api.grpc.v1.tasks.TaskDetailed\"&\n\x12\x43\x61ncelTasksRequest\x12\x10\n\x08task_ids\x18\x01 \x03(\t\"L\n\x13\x43\x61ncelTasksResponse\x12\x35\n\x05tasks\x18\x01 \x03(\x0b\x32&.armonik.api.grpc.v1.tasks.TaskSummary\"&\n\x13GetResultIdsRequest\x12\x0f\n\x07task_id\x18\x01 \x03(\t\"\xa1\x01\n\x14GetResultIdsResponse\x12S\n\x0ctask_results\x18\x01 \x03(\x0b\x32=.armonik.api.grpc.v1.tasks.GetResultIdsResponse.MapTaskResult\x1a\x34\n\rMapTaskResult\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x12\n\nresult_ids\x18\x02 \x03(\t\"P\n\x19\x43ountTasksByStatusRequest\x12\x33\n\x07\x66ilters\x18\x01 \x01(\x0b\x32\".armonik.api.grpc.v1.tasks.Filters\"N\n\x1a\x43ountTasksByStatusResponse\x12\x30\n\x06status\x18\x01 \x03(\x0b\x32 .armonik.api.grpc.v1.StatusCount\"\xca\x02\n\x12SubmitTasksRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x36\n\x0ctask_options\x18\x02 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12R\n\x0etask_creations\x18\x03 \x03(\x0b\x32:.armonik.api.grpc.v1.tasks.SubmitTasksRequest.TaskCreation\x1a\x93\x01\n\x0cTaskCreation\x12\x1c\n\x14\x65xpected_output_keys\x18\x01 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x02 \x03(\t\x12\x12\n\npayload_id\x18\x03 \x01(\t\x12\x36\n\x0ctask_options\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\"\xcb\x01\n\x13SubmitTasksResponse\x12K\n\ntask_infos\x18\x01 \x03(\x0b\x32\x37.armonik.api.grpc.v1.tasks.SubmitTasksResponse.TaskInfo\x1ag\n\x08TaskInfo\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x1b\n\x13\x65xpected_output_ids\x18\x02 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x03 \x03(\t\x12\x12\n\npayload_id\x18\x04 \x01(\tB\x1c\xaa\x02\x19\x41rmoniK.Api.gRPC.V1.Tasksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tasks_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\031ArmoniK.Api.gRPC.V1.Tasks'
  _globals['_TASKDETAILED']._serialized_start=212
  _globals['_TASKDETAILED']._serialized_end=1242
  _globals['_TASKDETAILED_OUTPUT']._serialized_start=1202
  _globals['_TASKDETAILED_OUTPUT']._serialized_end=1242
  _globals['_TASKSUMMARY']._serialized_start=1245
  _globals['_TASKSUMMARY']._serialized_end=2207
  _globals['_LISTTASKSREQUEST']._serialized_start=2210
  _globals['_LISTTASKSREQUEST']._serialized_end=2531
  _globals['_LISTTASKSREQUEST_SORT']._serialized_start=2402
  _globals['_LISTTASKSREQUEST_SORT']._serialized_end=2531
  _globals['_LISTTASKSRESPONSE']._serialized_start=2533
  _globals['_LISTTASKSRESPONSE']._serialized_end=2655
  _globals['_LISTTASKSDETAILEDRESPONSE']._serialized_start=2658
  _globals['_LISTTASKSDETAILEDRESPONSE']._serialized_end=2789
  _globals['_GETTASKREQUEST']._serialized_start=2791
  _globals['_GETTASKREQUEST']._serialized_end=2824
  _globals['_GETTASKRESPONSE']._serialized_start=2826
  _globals['_GETTASKRESPONSE']._serialized_end=2898
  _globals['_CANCELTASKSREQUEST']._serialized_start=2900
  _globals['_CANCELTASKSREQUEST']._serialized_end=2938
  _globals['_CANCELTASKSRESPONSE']._serialized_start=2940
  _globals['_CANCELTASKSRESPONSE']._serialized_end=3016
  _globals['_GETRESULTIDSREQUEST']._serialized_start=3018
  _globals['_GETRESULTIDSREQUEST']._serialized_end=3056
  _globals['_GETRESULTIDSRESPONSE']._serialized_start=3059
  _globals['_GETRESULTIDSRESPONSE']._serialized_end=3220
  _globals['_GETRESULTIDSRESPONSE_MAPTASKRESULT']._serialized_start=3168
  _globals['_GETRESULTIDSRESPONSE_MAPTASKRESULT']._serialized_end=3220
  _globals['_COUNTTASKSBYSTATUSREQUEST']._serialized_start=3222
  _globals['_COUNTTASKSBYSTATUSREQUEST']._serialized_end=3302
  _globals['_COUNTTASKSBYSTATUSRESPONSE']._serialized_start=3304
  _globals['_COUNTTASKSBYSTATUSRESPONSE']._serialized_end=3382
  _globals['_SUBMITTASKSREQUEST']._serialized_start=3385
  _globals['_SUBMITTASKSREQUEST']._serialized_end=3715
  _globals['_SUBMITTASKSREQUEST_TASKCREATION']._serialized_start=3568
  _globals['_SUBMITTASKSREQUEST_TASKCREATION']._serialized_end=3715
  _globals['_SUBMITTASKSRESPONSE']._serialized_start=3718
  _globals['_SUBMITTASKSRESPONSE']._serialized_end=3921
  _globals['_SUBMITTASKSRESPONSE_TASKINFO']._serialized_start=3818
  _globals['_SUBMITTASKSRESPONSE_TASKINFO']._serialized_end=3921
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: applications_common.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import applications_fields_pb2 as applications__fields__pb2
from . import applications_filters_pb2 as applications__filters__pb2
from . import sort_direction_pb2 as sort__direction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x61pplications_common.proto\x12 armonik.api.grpc.v1.applications\x1a\x19\x61pplications_fields.proto\x1a\x1a\x61pplications_filters.proto\x1a\x14sort_direction.proto\"S\n\x0e\x41pplicationRaw\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x11\n\tnamespace\x18\x03 \x01(\t\x12\x0f\n\x07service\x18\x04 \x01(\t\"\xd7\x02\n\x17ListApplicationsRequest\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12:\n\x07\x66ilters\x18\x03 \x01(\x0b\x32).armonik.api.grpc.v1.applications.Filters\x12L\n\x04sort\x18\x04 \x01(\x0b\x32>.armonik.api.grpc.v1.applications.ListApplicationsRequest.Sort\x1a\x90\x01\n\x04Sort\x12\x42\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x32.armonik.api.grpc.v1.applications.ApplicationField\x12\x44\n\tdirection\x18\x02 \x01(\x0e\x32\x31.armonik.api.grpc.v1.sort_direction.SortDirection\"\x92\x01\n\x18ListApplicationsResponse\x12\x46\n\x0c\x61pplications\x18\x01 \x03(\x0b\x32\x30.armonik.api.grpc.v1.applications.ApplicationRaw\x12\x0c\n\x04page\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\x42#\xaa\x02 ArmoniK.Api.gRPC.V1.Applicationsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'applications_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002 ArmoniK.Api.gRPC.V1.Applications'
  _globals['_APPLICATIONRAW']._serialized_start=140
  _globals['_APPLICATIONRAW']._serialized_end=223
  _globals['_LISTAPPLICATIONSREQUEST']._serialized_start=226
  _globals['_LISTAPPLICATIONSREQUEST']._serialized_end=569
  _globals['_LISTAPPLICATIONSREQUEST_SORT']._serialized_start=425
  _globals['_LISTAPPLICATIONSREQUEST_SORT']._serialized_end=569
  _globals['_LISTAPPLICATIONSRESPONSE']._serialized_start=572
  _globals['_LISTAPPLICATIONSRESPONSE']._serialized_end=718
# @@protoc_insertion_point(module_scope)



from .enumwrapper import (
    Direction,
    EventTypes,
    HealthCheckStatus,
    ResultStatus,
    ServiceHealthCheckStatus,
    SessionStatus,
    TaskStatus,
)
from .events import (
    Event,
    NewResultEvent,
    NewTaskEvent,
    ResultOwnerUpdateEvent,
    ResultStatusUpdateEvent,
    TaskStatusUpdateEvent,
)
from .filter import Filter, StatusFilter, StringFilter
from .helpers import (
    batched,
    datetime_to_timestamp,
    duration_to_timedelta,
    get_task_filter,
    timedelta_to_duration,
    timestamp_to_datetime,
)
from .objects import (
    Output,
    Partition,
    Result,
    ResultAvailability,
    Session,
    Task,
    TaskDefinition,
    TaskOptions,
)

__all__ = [
    "datetime_to_timestamp",
    "timestamp_to_datetime",
    "duration_to_timedelta",
    "timedelta_to_duration",
    "get_task_filter",
    "batched",
    "Task",
    "TaskDefinition",
    "TaskOptions",
    "Output",
    "ResultAvailability",
    "Session",
    "Result",
    "Partition",
    "HealthCheckStatus",
    "TaskStatus",
    "Direction",
    "SessionStatus",
    "ResultStatus",
    "EventTypes",
    # Include all names from events module
    "Filter",
    "StringFilter",
    "StatusFilter",
    "ServiceHealthCheckStatus",
    "NewResultEvent",
    "NewTaskEvent",
    "ResultOwnerUpdateEvent",
    "ResultStatusUpdateEvent",
    "TaskStatusUpdateEvent",
    "Event",
]



import datetime

from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKSessions, SessionFieldFilter
from armonik.common import Session, SessionStatus, TaskOptions


class TestArmoniKSessions:
    def test_create_session(self):
        sessions_client: ArmoniKSessions = get_client("Sessions")
        default_task_options = TaskOptions(
            max_duration=datetime.timedelta(seconds=1), priority=1, max_retries=1
        )
        session_id = sessions_client.create_session(default_task_options)

        assert rpc_called("Sessions", "CreateSession")
        assert session_id == "session-id"

    def test_get_session(self):
        sessions_client: ArmoniKSessions = get_client("Sessions")
        session = sessions_client.get_session("session-id")

        assert rpc_called("Sessions", "GetSession")
        assert isinstance(session, Session)
        assert session.session_id == "session-id"
        assert session.status == SessionStatus.CANCELLED
        assert session.partition_ids == []
        assert session.options == TaskOptions(
            max_duration=datetime.timedelta(0),
            priority=0,
            max_retries=0,
            partition_id="",
            application_name="",
            application_version="",
            application_namespace="",
            application_service="",
            engine_type="",
            options={},
        )
        assert session.created_at == datetime.datetime(
            1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc
        )
        assert session.cancelled_at == datetime.datetime(
            1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc
        )
        assert session.duration == datetime.timedelta(0)

    def test_list_session_no_filter(self):
        sessions_client: ArmoniKSessions = get_client("Sessions")
        num, sessions = sessions_client.list_sessions()

        assert rpc_called("Sessions", "ListSessions")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert sessions == []

    def test_list_session_with_filter(self):
        sessions_client: ArmoniKSessions = get_client("Sessions")
        num, sessions = sessions_client.list_sessions(
            SessionFieldFilter.STATUS == SessionStatus.RUNNING
        )

        assert rpc_called("Sessions", "ListSessions", 2)
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert sessions == []

    def test_cancel_session(self):
        sessions_client: ArmoniKSessions = get_client("Sessions")
        sessions_client.cancel_session("session-id")

        assert rpc_called("Sessions", "CancelSession")

    def test_pause_session(self):
        session_client: ArmoniKSessions = get_client("Sessions")
        session_client.pause_session("session-id")

        assert rpc_called("Sessions", "PauseSession")

    def test_resume_session(self):
        session_client: ArmoniKSessions = get_client("Sessions")
        session_client.resume_session("session-id")

        assert rpc_called("Sessions", "ResumeSession")

    def test_purge_session(self):
        session_client: ArmoniKSessions = get_client("Sessions")
        session_client.purge_session("session-id")

        assert rpc_called("Sessions", "PurgeSession")

    def test_delete_session(self):
        session_client: ArmoniKSessions = get_client("Sessions")
        session_client.delete_session("session-id")

        assert rpc_called("Sessions", "DeleteSession")

    def test_stop_submission_session(self):
        session_client: ArmoniKSessions = get_client("Sessions")
        session_client.stop_submission_session("session-id", True, True)

        assert rpc_called("Sessions", "StopSubmission")

    def test_service_fully_implemented(self):
        assert all_rpc_called("Sessions")



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class HealthStatusEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    HEALTH_STATUS_ENUM_UNSPECIFIED: _ClassVar[HealthStatusEnum]
    HEALTH_STATUS_ENUM_HEALTHY: _ClassVar[HealthStatusEnum]
    HEALTH_STATUS_ENUM_DEGRADED: _ClassVar[HealthStatusEnum]
    HEALTH_STATUS_ENUM_UNHEALTHY: _ClassVar[HealthStatusEnum]
HEALTH_STATUS_ENUM_UNSPECIFIED: HealthStatusEnum
HEALTH_STATUS_ENUM_HEALTHY: HealthStatusEnum
HEALTH_STATUS_ENUM_DEGRADED: HealthStatusEnum
HEALTH_STATUS_ENUM_UNHEALTHY: HealthStatusEnum

class CheckHealthRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class CheckHealthResponse(_message.Message):
    __slots__ = ("services",)
    class ServiceHealth(_message.Message):
        __slots__ = ("name", "message", "healthy")
        NAME_FIELD_NUMBER: _ClassVar[int]
        MESSAGE_FIELD_NUMBER: _ClassVar[int]
        HEALTHY_FIELD_NUMBER: _ClassVar[int]
        name: str
        message: str
        healthy: HealthStatusEnum
        def __init__(self, name: _Optional[str] = ..., message: _Optional[str] = ..., healthy: _Optional[_Union[HealthStatusEnum, str]] = ...) -> None: ...
    SERVICES_FIELD_NUMBER: _ClassVar[int]
    services: _containers.RepeatedCompositeFieldContainer[CheckHealthResponse.ServiceHealth]
    def __init__(self, services: _Optional[_Iterable[_Union[CheckHealthResponse.ServiceHealth, _Mapping]]] = ...) -> None: ...



from grpc import Channel

from ..protogen.client.health_checks_service_pb2_grpc import HealthChecksServiceStub
from ..protogen.common.health_checks_common_pb2 import (
    CheckHealthRequest,
    CheckHealthResponse,
)


class ArmoniKHealthChecks:
    def __init__(self, grpc_channel: Channel):
        """Result service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = HealthChecksServiceStub(grpc_channel)

    def check_health(self):
        response: CheckHealthResponse = self._client.CheckHealth(CheckHealthRequest())
        return {
            service.name: {"message": service.message, "status": service.healthy}
            for service in response.services
        }



from typing import List, Tuple, cast

from grpc import Channel

from ..common import Direction, Partition
from ..common.filter import Filter, NumberFilter
from ..protogen.client.partitions_service_pb2_grpc import PartitionsStub
from ..protogen.common.partitions_common_pb2 import (
    GetPartitionRequest,
    ListPartitionsRequest,
    ListPartitionsResponse,
)
from ..protogen.common.partitions_fields_pb2 import (
    PARTITION_RAW_ENUM_FIELD_PRIORITY,
    PartitionField,
    PartitionRawField,
)
from ..protogen.common.partitions_filters_pb2 import (
    FilterField as rawFilterField,
)
from ..protogen.common.partitions_filters_pb2 import (
    Filters as rawFilters,
)
from ..protogen.common.partitions_filters_pb2 import (
    FiltersAnd as rawFiltersAnd,
)
from ..protogen.common.sort_direction_pb2 import SortDirection


class PartitionFieldFilter:
    PRIORITY = NumberFilter(
        PartitionField(
            partition_raw_field=PartitionRawField(field=PARTITION_RAW_ENUM_FIELD_PRIORITY)
        ),
        rawFilters,
        rawFiltersAnd,
        rawFilterField,
    )


class ArmoniKPartitions:
    def __init__(self, grpc_channel: Channel):
        """Result service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = PartitionsStub(grpc_channel)

    def list_partitions(
        self,
        partition_filter: Filter | None = None,
        page: int = 0,
        page_size: int = 1000,
        sort_field: Filter = PartitionFieldFilter.PRIORITY,
        sort_direction: SortDirection = Direction.ASC,
    ) -> Tuple[int, List[Partition]]:
        """List partitions based on a filter.

        Args:
            partition_filter: Filter to apply when listing partitions
            page: page number to request, useful for pagination, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field to sort the resulting list by, defaults to the status
            sort_direction: direction of the sort, defaults to ascending

        Returns:
            A tuple containing :
            - The total number of results for the given filter
            - The obtained list of results
        """
        request = ListPartitionsRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, partition_filter.to_disjunction().to_message())
            if partition_filter
            else rawFilters(),
            sort=ListPartitionsRequest.Sort(
                field=cast(PartitionField, sort_field.field), direction=sort_direction
            ),
        )
        response: ListPartitionsResponse = self._client.ListPartitions(request)
        return response.total, [Partition.from_message(p) for p in response.partitions]

    def get_partition(self, partition_id: str) -> Partition:
        """Get a partition by its ID.

        Args:
            partition_id: The partition ID.

        Return:
            The partition summary.
        """
        return Partition.from_message(
            self._client.GetPartition(GetPartitionRequest(id=partition_id)).partition
        )



from abc import ABC
from typing import List

from dataclasses import dataclass

from .enumwrapper import TaskStatus, ResultStatus


class Event(ABC):
    @classmethod
    def from_raw_event(cls, raw_event):
        values = {}
        for raw_field in cls.__annotations__.keys():
            values[raw_field] = getattr(raw_event, raw_field)
        return cls(**values)


@dataclass
class TaskStatusUpdateEvent(Event):
    task_id: str
    status: TaskStatus


@dataclass
class ResultStatusUpdateEvent(Event):
    result_id: str
    status: ResultStatus


@dataclass
class ResultOwnerUpdateEvent(Event):
    result_id: str
    previous_owner_id: str
    current_owner_id: str


@dataclass
class NewTaskEvent(Event):
    task_id: str
    payload_id: str
    origin_task_id: str
    status: TaskStatus
    expected_output_keys: List[str]
    data_dependencies: List[str]
    retry_of_ids: List[str]
    parent_task_ids: List[str]


@dataclass
class NewResultEvent(Event):
    result_id: str
    owner_id: str
    status: ResultStatus



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: submitter_common.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import objects_pb2 as objects__pb2
from . import result_status_pb2 as result__status__pb2
from . import session_status_pb2 as session__status__pb2
from . import task_status_pb2 as task__status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16submitter_common.proto\x12\x1d\x61rmonik.api.grpc.v1.submitter\x1a\robjects.proto\x1a\x13result_status.proto\x1a\x14session_status.proto\x1a\x11task_status.proto\"=\n\x0bSessionList\x12.\n\x08sessions\x18\x01 \x03(\x0b\x32\x1c.armonik.api.grpc.v1.Session\"$\n\rSessionIdList\x12\x13\n\x0bsession_ids\x18\x01 \x03(\t\"l\n\x14\x43reateSessionRequest\x12=\n\x13\x64\x65\x66\x61ult_task_option\x18\x01 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12\x15\n\rpartition_ids\x18\x02 \x03(\t\"(\n\x12\x43reateSessionReply\x12\x12\n\nsession_id\x18\x01 \x01(\t\"\x9d\x01\n\x16\x43reateSmallTaskRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x36\n\x0ctask_options\x18\x02 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12\x37\n\rtask_requests\x18\x03 \x03(\x0b\x32 .armonik.api.grpc.v1.TaskRequest\"\xc9\x02\n\x16\x43reateLargeTaskRequest\x12Y\n\x0cinit_request\x18\x01 \x01(\x0b\x32\x41.armonik.api.grpc.v1.submitter.CreateLargeTaskRequest.InitRequestH\x00\x12\x39\n\tinit_task\x18\x02 \x01(\x0b\x32$.armonik.api.grpc.v1.InitTaskRequestH\x00\x12\x36\n\x0ctask_payload\x18\x03 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.DataChunkH\x00\x1aY\n\x0bInitRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x36\n\x0ctask_options\x18\x02 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptionsB\x06\n\x04type\"\xe6\x03\n\x0f\x43reateTaskReply\x12\x61\n\x14\x63reation_status_list\x18\x01 \x01(\x0b\x32\x41.armonik.api.grpc.v1.submitter.CreateTaskReply.CreationStatusListH\x00\x12\x0f\n\x05\x65rror\x18\x02 \x01(\tH\x00\x1ah\n\x08TaskInfo\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x1c\n\x14\x65xpected_output_keys\x18\x02 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x03 \x03(\t\x12\x12\n\npayload_id\x18\x04 \x01(\t\x1ay\n\x0e\x43reationStatus\x12L\n\ttask_info\x18\x01 \x01(\x0b\x32\x37.armonik.api.grpc.v1.submitter.CreateTaskReply.TaskInfoH\x00\x12\x0f\n\x05\x65rror\x18\x02 \x01(\tH\x00\x42\x08\n\x06Status\x1an\n\x12\x43reationStatusList\x12X\n\x11\x63reation_statuses\x18\x01 \x03(\x0b\x32=.armonik.api.grpc.v1.submitter.CreateTaskReply.CreationStatusB\n\n\x08Response\"\xb9\x03\n\nTaskFilter\x12G\n\x07session\x18\x01 \x01(\x0b\x32\x34.armonik.api.grpc.v1.submitter.TaskFilter.IdsRequestH\x00\x12\x44\n\x04task\x18\x03 \x01(\x0b\x32\x34.armonik.api.grpc.v1.submitter.TaskFilter.IdsRequestH\x00\x12M\n\x08included\x18\x04 \x01(\x0b\x32\x39.armonik.api.grpc.v1.submitter.TaskFilter.StatusesRequestH\x01\x12M\n\x08\x65xcluded\x18\x05 \x01(\x0b\x32\x39.armonik.api.grpc.v1.submitter.TaskFilter.StatusesRequestH\x01\x1a\x19\n\nIdsRequest\x12\x0b\n\x03ids\x18\x01 \x03(\t\x1aP\n\x0fStatusesRequest\x12=\n\x08statuses\x18\x01 \x03(\x0e\x32+.armonik.api.grpc.v1.task_status.TaskStatusB\x05\n\x03idsB\n\n\x08statuses\"\xa9\x02\n\rSessionFilter\x12\x10\n\x08sessions\x18\x01 \x03(\t\x12P\n\x08included\x18\x04 \x01(\x0b\x32<.armonik.api.grpc.v1.submitter.SessionFilter.StatusesRequestH\x00\x12P\n\x08\x65xcluded\x18\x05 \x01(\x0b\x32<.armonik.api.grpc.v1.submitter.SessionFilter.StatusesRequestH\x00\x1aV\n\x0fStatusesRequest\x12\x43\n\x08statuses\x18\x01 \x03(\x0e\x32\x31.armonik.api.grpc.v1.session_status.SessionStatusB\n\n\x08statuses\"(\n\x14GetTaskStatusRequest\x12\x10\n\x08task_ids\x18\x01 \x03(\t\"\xbf\x01\n\x12GetTaskStatusReply\x12O\n\x0bid_statuses\x18\x01 \x03(\x0b\x32:.armonik.api.grpc.v1.submitter.GetTaskStatusReply.IdStatus\x1aX\n\x08IdStatus\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12;\n\x06status\x18\x02 \x01(\x0e\x32+.armonik.api.grpc.v1.task_status.TaskStatus\"@\n\x16GetResultStatusRequest\x12\x12\n\nresult_ids\x18\x01 \x03(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\"\xc9\x01\n\x14GetResultStatusReply\x12Q\n\x0bid_statuses\x18\x01 \x03(\x0b\x32<.armonik.api.grpc.v1.submitter.GetResultStatusReply.IdStatus\x1a^\n\x08IdStatus\x12\x11\n\tresult_id\x18\x01 \x01(\t\x12?\n\x06status\x18\x02 \x01(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\"\x96\x01\n\x0bResultReply\x12\x30\n\x06result\x18\x01 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.DataChunkH\x00\x12/\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.TaskErrorH\x00\x12\x1c\n\x12not_completed_task\x18\x03 \x01(\tH\x00\x42\x06\n\x04type\"\x94\x01\n\x11\x41vailabilityReply\x12(\n\x02ok\x18\x01 \x01(\x0b\x32\x1a.armonik.api.grpc.v1.EmptyH\x00\x12/\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.TaskErrorH\x00\x12\x1c\n\x12not_completed_task\x18\x03 \x01(\tH\x00\x42\x06\n\x04type\"\x93\x01\n\x0bWaitRequest\x12\x39\n\x06\x66ilter\x18\x01 \x01(\x0b\x32).armonik.api.grpc.v1.submitter.TaskFilter\x12 \n\x18stop_on_first_task_error\x18\x02 \x01(\x08\x12\'\n\x1fstop_on_first_task_cancellation\x18\x03 \x01(\x08\"\xba\x01\n\x12WatchResultRequest\x12G\n\x0e\x66\x65tch_statuses\x18\x01 \x03(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\x12G\n\x0ewatch_statuses\x18\x02 \x03(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\x12\x12\n\nresult_ids\x18\x03 \x03(\t\"h\n\x11WatchResultStream\x12?\n\x06status\x18\x01 \x01(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\x12\x12\n\nresult_ids\x18\x02 \x03(\tB \xaa\x02\x1d\x41rmoniK.Api.gRPC.V1.Submitterb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'submitter_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\035ArmoniK.Api.gRPC.V1.Submitter'
  _globals['_SESSIONLIST']._serialized_start=134
  _globals['_SESSIONLIST']._serialized_end=195
  _globals['_SESSIONIDLIST']._serialized_start=197
  _globals['_SESSIONIDLIST']._serialized_end=233
  _globals['_CREATESESSIONREQUEST']._serialized_start=235
  _globals['_CREATESESSIONREQUEST']._serialized_end=343
  _globals['_CREATESESSIONREPLY']._serialized_start=345
  _globals['_CREATESESSIONREPLY']._serialized_end=385
  _globals['_CREATESMALLTASKREQUEST']._serialized_start=388
  _globals['_CREATESMALLTASKREQUEST']._serialized_end=545
  _globals['_CREATELARGETASKREQUEST']._serialized_start=548
  _globals['_CREATELARGETASKREQUEST']._serialized_end=877
  _globals['_CREATELARGETASKREQUEST_INITREQUEST']._serialized_start=780
  _globals['_CREATELARGETASKREQUEST_INITREQUEST']._serialized_end=869
  _globals['_CREATETASKREPLY']._serialized_start=880
  _globals['_CREATETASKREPLY']._serialized_end=1366
  _globals['_CREATETASKREPLY_TASKINFO']._serialized_start=1015
  _globals['_CREATETASKREPLY_TASKINFO']._serialized_end=1119
  _globals['_CREATETASKREPLY_CREATIONSTATUS']._serialized_start=1121
  _globals['_CREATETASKREPLY_CREATIONSTATUS']._serialized_end=1242
  _globals['_CREATETASKREPLY_CREATIONSTATUSLIST']._serialized_start=1244
  _globals['_CREATETASKREPLY_CREATIONSTATUSLIST']._serialized_end=1354
  _globals['_TASKFILTER']._serialized_start=1369
  _globals['_TASKFILTER']._serialized_end=1810
  _globals['_TASKFILTER_IDSREQUEST']._serialized_start=1684
  _globals['_TASKFILTER_IDSREQUEST']._serialized_end=1709
  _globals['_TASKFILTER_STATUSESREQUEST']._serialized_start=1711
  _globals['_TASKFILTER_STATUSESREQUEST']._serialized_end=1791
  _globals['_SESSIONFILTER']._serialized_start=1813
  _globals['_SESSIONFILTER']._serialized_end=2110
  _globals['_SESSIONFILTER_STATUSESREQUEST']._serialized_start=2012
  _globals['_SESSIONFILTER_STATUSESREQUEST']._serialized_end=2098
  _globals['_GETTASKSTATUSREQUEST']._serialized_start=2112
  _globals['_GETTASKSTATUSREQUEST']._serialized_end=2152
  _globals['_GETTASKSTATUSREPLY']._serialized_start=2155
  _globals['_GETTASKSTATUSREPLY']._serialized_end=2346
  _globals['_GETTASKSTATUSREPLY_IDSTATUS']._serialized_start=2258
  _globals['_GETTASKSTATUSREPLY_IDSTATUS']._serialized_end=2346
  _globals['_GETRESULTSTATUSREQUEST']._serialized_start=2348
  _globals['_GETRESULTSTATUSREQUEST']._serialized_end=2412
  _globals['_GETRESULTSTATUSREPLY']._serialized_start=2415
  _globals['_GETRESULTSTATUSREPLY']._serialized_end=2616
  _globals['_GETRESULTSTATUSREPLY_IDSTATUS']._serialized_start=2522
  _globals['_GETRESULTSTATUSREPLY_IDSTATUS']._serialized_end=2616
  _globals['_RESULTREPLY']._serialized_start=2619
  _globals['_RESULTREPLY']._serialized_end=2769
  _globals['_AVAILABILITYREPLY']._serialized_start=2772
  _globals['_AVAILABILITYREPLY']._serialized_end=2920
  _globals['_WAITREQUEST']._serialized_start=2923
  _globals['_WAITREQUEST']._serialized_end=3070
  _globals['_WATCHRESULTREQUEST']._serialized_start=3073
  _globals['_WATCHRESULTREQUEST']._serialized_end=3259
  _globals['_WATCHRESULTSTREAM']._serialized_start=3261
  _globals['_WATCHRESULTSTREAM']._serialized_end=3365
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timedelta
from typing import Dict, List, Optional

from ..protogen.common.objects_pb2 import (
    Empty,
)
from ..protogen.common.objects_pb2 import (
    Output as WorkerOutput,
)
from ..protogen.common.objects_pb2 import (
    TaskOptions as RawTaskOptions,
)
from ..protogen.common.partitions_common_pb2 import PartitionRaw
from ..protogen.common.result_status_pb2 import ResultStatus as RawResultStatus
from ..protogen.common.results_common_pb2 import ResultRaw
from ..protogen.common.session_status_pb2 import SessionStatus as RawSessionStatus
from ..protogen.common.sessions_common_pb2 import SessionRaw
from ..protogen.common.task_status_pb2 import TaskStatus as RawTaskStatus
from ..protogen.common.tasks_common_pb2 import TaskDetailed
from .enumwrapper import ResultStatus, SessionStatus, TaskStatus
from .helpers import duration_to_timedelta, timedelta_to_duration, timestamp_to_datetime


@dataclass()
class TaskOptions:
    max_duration: timedelta
    priority: int
    max_retries: int
    partition_id: Optional[str] = None
    application_name: Optional[str] = None
    application_version: Optional[str] = None
    application_namespace: Optional[str] = None
    application_service: Optional[str] = None
    engine_type: Optional[str] = None
    options: Dict[str, str] = field(default_factory=dict)

    @classmethod
    def from_message(cls, task_options):
        return cls(
            max_duration=duration_to_timedelta(task_options.max_duration),
            max_retries=task_options.max_retries,
            priority=task_options.priority,
            partition_id=task_options.partition_id,
            application_name=task_options.application_name,
            application_version=task_options.application_version,
            application_namespace=task_options.application_namespace,
            application_service=task_options.application_service,
            engine_type=task_options.engine_type,
            options=task_options.options,
        )

    def to_message(self) -> RawTaskOptions:
        return RawTaskOptions(
            max_duration=timedelta_to_duration(self.max_duration),
            max_retries=self.max_retries,
            priority=self.priority,
            partition_id=self.partition_id,
            application_name=self.application_name,
            application_version=self.application_version,
            application_namespace=self.application_namespace,
            application_service=self.application_service,
            engine_type=self.engine_type,
            options=self.options,
        )


@dataclass()
class Output:
    error: Optional[str] = None

    @property
    def success(self) -> bool:
        return self.error is None

    def to_message(self):
        if self.error is None:
            return WorkerOutput(ok=Empty())
        return WorkerOutput(error=WorkerOutput.Error(details=self.error))


@dataclass()
class TaskDefinition:
    payload_id: str = field(default_factory=str)
    payload: bytes = field(default_factory=bytes)
    expected_output_ids: List[str] = field(default_factory=list)
    data_dependencies: List[str] = field(default_factory=list)
    options: Optional[TaskOptions] = None

    def __post_init__(self):
        if len(self.expected_output_ids) <= 0:
            raise ValueError("expected_output_ids must be not be empty")


@dataclass()
class Task:
    id: Optional[str] = None
    session_id: Optional[str] = None
    owner_pod_id: Optional[str] = None
    parent_task_ids: List[str] = field(default_factory=list)
    data_dependencies: List[str] = field(default_factory=list)
    expected_output_ids: List[str] = field(default_factory=list)
    retry_of_ids: List[str] = field(default_factory=list)
    status: RawTaskStatus = TaskStatus.UNSPECIFIED
    payload_id: Optional[str] = None
    status_message: Optional[str] = None
    options: Optional[TaskOptions] = None
    created_at: Optional[datetime] = None
    submitted_at: Optional[datetime] = None
    started_at: Optional[datetime] = None
    ended_at: Optional[datetime] = None
    pod_ttl: Optional[datetime] = None
    output: Optional[Output] = None
    pod_hostname: Optional[str] = None
    received_at: Optional[datetime] = None
    acquired_at: Optional[datetime] = None

    def refresh(self, task_client) -> None:
        """Refresh the fields of this task object by using the given task client

        Args:
            task_client: ArmoniKTasks client
        """
        result: "Task" = task_client.get_task(self.id)
        self.session_id = result.session_id
        self.owner_pod_id = result.owner_pod_id
        self.parent_task_ids = result.parent_task_ids
        self.data_dependencies = result.data_dependencies
        self.expected_output_ids = result.expected_output_ids
        self.retry_of_ids = result.retry_of_ids
        self.status = result.status
        self.status_message = result.status_message
        self.options = result.options
        self.created_at = result.created_at
        self.submitted_at = result.submitted_at
        self.started_at = result.started_at
        self.ended_at = result.ended_at
        self.pod_ttl = result.pod_ttl
        self.output = result.output
        self.pod_hostname = result.pod_hostname
        self.received_at = result.received_at
        self.acquired_at = result.acquired_at
        self.is_init = True

    @classmethod
    def from_message(cls, task_raw: TaskDetailed) -> "Task":
        return cls(
            id=task_raw.id,
            session_id=task_raw.session_id,
            owner_pod_id=task_raw.owner_pod_id,
            parent_task_ids=list(task_raw.parent_task_ids),
            data_dependencies=list(task_raw.data_dependencies),
            expected_output_ids=list(task_raw.expected_output_ids),
            retry_of_ids=list(task_raw.retry_of_ids),
            status=task_raw.status,
            status_message=task_raw.status_message,
            options=TaskOptions.from_message(task_raw.options),
            created_at=timestamp_to_datetime(task_raw.created_at),
            submitted_at=timestamp_to_datetime(task_raw.submitted_at),
            started_at=timestamp_to_datetime(task_raw.started_at),
            ended_at=timestamp_to_datetime(task_raw.ended_at),
            pod_ttl=timestamp_to_datetime(task_raw.pod_ttl),
            output=Output(error=(task_raw.output.error if not task_raw.output.success else None)),
            pod_hostname=task_raw.pod_hostname,
            received_at=timestamp_to_datetime(task_raw.received_at),
            acquired_at=timestamp_to_datetime(task_raw.acquired_at),
        )


@dataclass()
class ResultAvailability:
    errors: List[str] = field(default_factory=list)

    def is_available(self) -> bool:
        return len(self.errors) == 0


@dataclass
class Session:
    session_id: Optional[str] = None
    status: RawSessionStatus = SessionStatus.UNSPECIFIED
    client_submission: Optional[bool] = None
    worker_submission: Optional[bool] = None
    partition_ids: List[str] = field(default_factory=list)
    options: Optional[TaskOptions] = None
    created_at: Optional[datetime] = None
    cancelled_at: Optional[datetime] = None
    purged_at: Optional[datetime] = None
    deleted_at: Optional[datetime] = None
    duration: Optional[timedelta] = None

    @classmethod
    def from_message(cls, session_raw: SessionRaw) -> "Session":
        return cls(
            session_id=session_raw.session_id,
            status=session_raw.status,
            client_submission=session_raw.client_submission,
            worker_submission=session_raw.worker_submission,
            partition_ids=list(session_raw.partition_ids),
            options=TaskOptions.from_message(session_raw.options),
            created_at=timestamp_to_datetime(session_raw.created_at),
            cancelled_at=timestamp_to_datetime(session_raw.cancelled_at),
            purged_at=timestamp_to_datetime(session_raw.purged_at),
            deleted_at=timestamp_to_datetime(session_raw.deleted_at),
            duration=duration_to_timedelta(session_raw.duration),
        )


@dataclass
class Result:
    session_id: Optional[str] = None
    name: Optional[str] = None
    owner_task_id: Optional[str] = None
    status: RawResultStatus = ResultStatus.UNSPECIFIED
    created_at: Optional[datetime] = None
    completed_at: Optional[datetime] = None
    result_id: Optional[str] = None
    size: Optional[int] = None

    @classmethod
    def from_message(cls, result_raw: ResultRaw) -> "Result":
        return cls(
            session_id=result_raw.session_id,
            name=result_raw.name,
            owner_task_id=result_raw.owner_task_id,
            status=result_raw.status,
            created_at=timestamp_to_datetime(result_raw.created_at),
            completed_at=timestamp_to_datetime(result_raw.completed_at),
            result_id=result_raw.result_id,
            size=result_raw.size,
        )


@dataclass
class Partition:
    id: str
    parent_partition_ids: List[str]
    pod_reserved: int
    pod_max: int
    pod_configuration: Dict[str, str]
    preemption_percentage: int
    priority: int

    @classmethod
    def from_message(cls, partition_raw: PartitionRaw) -> "Partition":
        return cls(
            id=partition_raw.id,
            parent_partition_ids=partition_raw.parent_partition_ids,
            pod_reserved=partition_raw.pod_reserved,
            pod_max=partition_raw.pod_max,
            pod_configuration=partition_raw.pod_configuration,
            preemption_percentage=partition_raw.preemption_percentage,
            priority=partition_raw.priority,
        )



from __future__ import annotations

import json
from abc import abstractmethod
from typing import Any, Dict, List, Optional, Type

import google.protobuf.duration_pb2 as duration
import google.protobuf.timestamp_pb2 as timestamp
from google.protobuf.message import Message

from ..protogen.common.filters_common_pb2 import (
    FILTER_ARRAY_OPERATOR_CONTAINS,
    FILTER_ARRAY_OPERATOR_NOT_CONTAINS,
    FILTER_BOOLEAN_OPERATOR_IS,
    FILTER_DATE_OPERATOR_AFTER,
    FILTER_DATE_OPERATOR_AFTER_OR_EQUAL,
    FILTER_DATE_OPERATOR_BEFORE,
    FILTER_DATE_OPERATOR_BEFORE_OR_EQUAL,
    FILTER_DATE_OPERATOR_EQUAL,
    FILTER_DATE_OPERATOR_NOT_EQUAL,
    FILTER_DURATION_OPERATOR_EQUAL,
    FILTER_DURATION_OPERATOR_LONGER_THAN,
    FILTER_DURATION_OPERATOR_LONGER_THAN_OR_EQUAL,
    FILTER_DURATION_OPERATOR_NOT_EQUAL,
    FILTER_DURATION_OPERATOR_SHORTER_THAN,
    FILTER_DURATION_OPERATOR_SHORTER_THAN_OR_EQUAL,
    FILTER_NUMBER_OPERATOR_EQUAL,
    FILTER_NUMBER_OPERATOR_GREATER_THAN,
    FILTER_NUMBER_OPERATOR_GREATER_THAN_OR_EQUAL,
    FILTER_NUMBER_OPERATOR_LESS_THAN,
    FILTER_NUMBER_OPERATOR_LESS_THAN_OR_EQUAL,
    FILTER_NUMBER_OPERATOR_NOT_EQUAL,
    FILTER_STATUS_OPERATOR_EQUAL,
    FILTER_STATUS_OPERATOR_NOT_EQUAL,
    FILTER_STRING_OPERATOR_CONTAINS,
    FILTER_STRING_OPERATOR_ENDS_WITH,
    FILTER_STRING_OPERATOR_EQUAL,
    FILTER_STRING_OPERATOR_NOT_CONTAINS,
    FILTER_STRING_OPERATOR_NOT_EQUAL,
    FILTER_STRING_OPERATOR_STARTS_WITH,
    FilterArray,
    FilterBoolean,
    FilterDate,
    FilterDuration,
    FilterNumber,
    FilterString,
)


class Filter:
    """
    Filter for use with ArmoniK

    Attributes:
        eq_: equality raw Api operator
        ne_: inequality raw Api operator
        lt_: less than raw Api operator
        le_: less or equal raw Api operator
        gt_: greater than raw Api operator
        ge_: greater or equal raw Api operator
        contains_: contains raw Api operator
        notcontains_: not contains raw Api operator
        value_type_: expected type for the value to test against in this filter

        field: field of the filter if it's a simple filter
        message_type: Api message type of the filter
        inner_message_type: Api message type of the inner filter (with value and operator)
        conjunction_type: Type of the conjunction for this filter
        value: value to test against in this filter if it's a simple filter
        operator: operator to apply for this filter if it's a simple filter
    """

    eq_ = None
    ne_ = None
    lt_ = None
    le_ = None
    gt_ = None
    ge_ = None
    contains_ = None
    notcontains_ = None
    value_type_ = None

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]],
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        self._filters: List[List["Filter"]] = [[]] if filters is None else filters
        self.field = field
        self.message_type = message_type
        self.conjunction_type = conjunction_message_type
        self.disjunction_type = disjunction_message_type
        self.inner_message_type = inner_message_type
        self.value = value
        self.operator = operator

    def is_true_conjunction(self) -> bool:
        """
        Tests whether the filter is a conjunction (logical and)
        Note : This will only output true if it's an actual conjunction with multiple filters and no disjunction
        """
        return self.message_type == self.conjunction_type or (
            len(self._filters) == 1 and len(self._filters[0]) > 1
        )

    def is_true_disjunction(self) -> bool:
        """
        Tests whether the filter is a disjunction (logical or)
        Note : This will only output true if it's an actual disjunction with multiple filters
        """
        return len(self._filters) > 1

    def to_disjunction(self) -> Filter:
        """
        Converts the filter into a disjunction

        """
        if self.is_true_disjunction():
            return self
        if self.is_true_conjunction():
            return Filter(
                None,
                self.disjunction_type,
                self.conjunction_type,
                self.disjunction_type,
                None,
                self._filters,
            )
        return Filter(
            None,
            self.disjunction_type,
            self.conjunction_type,
            self.disjunction_type,
            None,
            [[self]],
        )

    def __and__(self, other: "Filter") -> "Filter":
        if not isinstance(other, Filter):
            msg = f"Cannot create a conjunction between Filter and {other.__class__.__name__}"
            raise Exception(msg)
        if self.is_true_disjunction() or other.is_true_disjunction():
            raise Exception("Cannot make a conjunction of disjunctions")
        if self.conjunction_type != other.conjunction_type:
            raise Exception("Conjunction types are different")
        return Filter(
            None,
            self.disjunction_type,
            self.conjunction_type,
            self.conjunction_type,
            None,
            [self.to_disjunction()._filters[0] + other.to_disjunction()._filters[0]],
        )

    def __mul__(self, other: Filter) -> "Filter":
        return self & other

    def __or__(self, other: "Filter") -> "Filter":
        if not isinstance(other, Filter):
            msg = f"Cannot create a conjunction between Filter and {other.__class__.__name__}"
            raise Exception(msg)
        if self.disjunction_type != other.disjunction_type:
            raise Exception("Disjunction types are different")
        return Filter(
            None,
            self.disjunction_type,
            self.conjunction_type,
            self.disjunction_type,
            None,
            self.to_disjunction()._filters + other.to_disjunction()._filters,
        )

    def __add__(self, other: "Filter") -> "Filter":
        return self | other

    def __eq__(self, value) -> Filter:
        return self._check(value, self.__class__.eq_, "==")

    def __ne__(self, value) -> Filter:
        return self._check(value, self.__class__.ne_, "!=")

    def __lt__(self, value) -> Filter:
        return self._check(value, self.__class__.lt_, "<")

    def __le__(self, value) -> Filter:
        return self._check(value, self.__class__.le_, "<=")

    def __gt__(self, value) -> Filter:
        return self._check(value, self.__class__.gt_, ">")

    def __ge__(self, value) -> Filter:
        return self._check(value, self.__class__.ge_, ">=")

    def contains(self, value) -> Filter:
        return self._check(value, self.__class__.contains_, "contains")

    def __invert__(self) -> Filter:
        """
        Inverts the test

        Returns:
            Filter with the test being inverted
        """
        if self.operator is None:
            if self.is_true_conjunction() or self.is_true_disjunction():
                raise Exception("Cannot invert conjunctions or disjunctions")
            msg = f"Cannot invert None operator in class {self.__class__.__name__} for field {str(self.field)}"
            raise Exception(msg)
        if self.operator == self.__class__.eq_:
            return self.__ne__(self.value)
        if self.operator == self.__class__.ne_:
            return self.__eq__(self.value)
        if self.operator == self.__class__.lt_:
            return self.__ge__(self.value)
        if self.operator == self.__class__.le_:
            return self.__gt__(self.value)
        if self.operator == self.__class__.gt_:
            return self.__le__(self.value)
        if self.operator == self.__class__.ge_:
            return self.__lt__(self.value)
        if self.operator == self.__class__.contains_:
            return self._check(self.value, self.__class__.notcontains_, "not_contains")
        if self.operator == self.__class__.notcontains_:
            return self.contains(self.value)
        msg = f"{self.__class__.__name__} operator {str(self.operator)} for field {str(self.field)} has no inverted equivalent"
        raise Exception(msg)

    def __neg__(self) -> "Filter":
        return ~self

    def to_dict(self) -> Dict:
        rep = {}
        if self.is_true_disjunction():
            rep["or"] = [{"and": [f.to_dict() for f in conj]} for conj in self._filters]
            return rep
        if self.is_true_conjunction():
            rep["and"] = [f.to_dict() for f in self._filters[0]]
            return rep
        if len(self._filters) == 1 and len(self._filters[0]) == 1:
            return self._filters[0][0].to_dict()
        return {
            "field": str(self.field),
            "value": str(self.value),
            "operator": str(self.operator),
        }

    def __str__(self) -> str:
        return json.dumps(self.to_dict())

    def _verify_value(self, value):
        """
        Checks if the value is of the expected type
        Args:
            value: Value to test

        Raises:
            Exception if value is not of the expected type

        """
        if self.__class__.value_type_ is None or isinstance(value, self.__class__.value_type_):
            return
        msg = f"Expected value type {str(self.__class__.value_type_)} for field {str(self.field)}, got {str(type(value))} instead"
        raise Exception(msg)

    def _check(self, value: Any, operator: Any, operator_str: str = "") -> "Filter":
        """
        Internal function to create a new filter from the current filter with a different value and/or operator
        Args:
            value: Value of the new filter
            operator: Operator of the new filter
            operator_str: Optional string for error message clarification

        Returns:
            new filter with the given value/operator

        Raises:
            NotImplementedError if the given operator is not available for the given class
        """
        if self.is_true_conjunction() or self.is_true_disjunction():
            raise Exception("Cannot apply operator to a disjunction or a conjunction")
        self._verify_value(value)
        if operator is None:
            msg = f"Operator {operator_str} is not available for {self.__class__.__name__}"
            raise NotImplementedError(msg)
        return self.__class__(
            self.field,
            self.disjunction_type,
            self.conjunction_type,
            self.message_type,
            self.inner_message_type,
            self._filters,
            value,
            operator,
        )

    @abstractmethod
    def to_basic_message(self) -> Message:
        pass

    def to_message(self) -> Message:
        def to_conjunction_message(conj: List[Filter]) -> Message:
            conj_raw = self.conjunction_type()
            getattr(conj_raw, "and").extend([f.to_basic_message() for f in conj])
            return conj_raw

        if self.message_type == self.disjunction_type:
            raw = self.to_disjunction().disjunction_type()
            getattr(raw, "or").extend([to_conjunction_message(conj) for conj in self._filters])
            return raw
        if self.message_type == self.conjunction_type:
            return to_conjunction_message(self.to_disjunction()._filters[0])
        return self.to_basic_message()


class StringFilter(Filter):
    """
    Filter for string comparisons
    """

    eq_ = FILTER_STRING_OPERATOR_EQUAL
    ne_ = FILTER_STRING_OPERATOR_NOT_EQUAL
    contains_ = FILTER_STRING_OPERATOR_CONTAINS
    notcontains_ = FILTER_STRING_OPERATOR_NOT_CONTAINS
    value_type_ = str

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterString,
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def startswith(self, value: str) -> "StringFilter":
        return self._check(value, FILTER_STRING_OPERATOR_STARTS_WITH, "startswith")

    def endswith(self, value: str) -> "StringFilter":
        return self._check(value, FILTER_STRING_OPERATOR_ENDS_WITH, "endswith")

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_string=self.inner_message_type(value=self.value, operator=self.operator),
        )

    def __repr__(self) -> str:
        return f'{str(self.field)} {str(self.operator)} "{str(self.value)}"'


class StatusFilter(Filter):
    """
    Filter for status comparison
    """

    eq_ = FILTER_STATUS_OPERATOR_EQUAL
    ne_ = FILTER_STATUS_OPERATOR_NOT_EQUAL

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Type[Message],
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_status=self.inner_message_type(value=self.value, operator=self.operator),
        )


class DateFilter(Filter):
    """
    Filter for timestamp comparison
    """

    eq_ = FILTER_DATE_OPERATOR_EQUAL
    ne_ = FILTER_DATE_OPERATOR_NOT_EQUAL
    lt_ = FILTER_DATE_OPERATOR_BEFORE
    le_ = FILTER_DATE_OPERATOR_BEFORE_OR_EQUAL
    gt_ = FILTER_DATE_OPERATOR_AFTER
    ge_ = FILTER_DATE_OPERATOR_AFTER_OR_EQUAL
    value_type = timestamp.Timestamp

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterDate,
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_date=self.inner_message_type(value=self.value, operator=self.operator),
        )


class NumberFilter(Filter):
    """
    Filter for int comparison
    """

    eq_ = FILTER_NUMBER_OPERATOR_EQUAL
    ne_ = FILTER_NUMBER_OPERATOR_NOT_EQUAL
    lt_ = FILTER_NUMBER_OPERATOR_LESS_THAN
    le_ = FILTER_NUMBER_OPERATOR_LESS_THAN_OR_EQUAL
    gt_ = FILTER_NUMBER_OPERATOR_GREATER_THAN
    ge_ = FILTER_NUMBER_OPERATOR_GREATER_THAN_OR_EQUAL
    value_type_ = int

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterNumber,
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_number=self.inner_message_type(value=self.value, operator=self.operator),
        )


class BooleanFilter(Filter):
    """
    Filter for boolean comparison
    """

    eq_ = FILTER_BOOLEAN_OPERATOR_IS
    value_type_ = bool

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterBoolean,
        filters: Optional[List[List["Filter"]]] = None,
        value=True,
        operator=FILTER_BOOLEAN_OPERATOR_IS,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def __ne__(self, value: bool) -> "BooleanFilter":
        return self.__eq__(not value)

    def __invert__(self) -> "BooleanFilter":
        return self.__eq__(not self.value)

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_boolean=self.inner_message_type(value=self.value, operator=self.operator),
        )


class ArrayFilter(Filter):
    """
    Filter for array comparisons
    """

    contains_ = FILTER_ARRAY_OPERATOR_CONTAINS
    notcontains_ = FILTER_ARRAY_OPERATOR_NOT_CONTAINS
    value_type_ = str

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterArray,
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_array=self.inner_message_type(value=self.value, operator=self.operator),
        )


class DurationFilter(Filter):
    """
    Filter for duration comparison
    """

    eq_ = FILTER_DURATION_OPERATOR_EQUAL
    ne_ = FILTER_DURATION_OPERATOR_NOT_EQUAL
    lt_ = FILTER_DURATION_OPERATOR_SHORTER_THAN
    le_ = FILTER_DURATION_OPERATOR_SHORTER_THAN_OR_EQUAL
    gt_ = FILTER_DURATION_OPERATOR_LONGER_THAN
    ge_ = FILTER_DURATION_OPERATOR_LONGER_THAN_OR_EQUAL
    value_type_ = duration.Duration

    def __init__(
        self,
        field: Optional[Message],
        disjunction_message_type: Type[Message],
        conjunction_message_type: Type[Message],
        message_type: Type[Message],
        inner_message_type: Optional[Type[Message]] = FilterDuration,
        filters: Optional[List[List["Filter"]]] = None,
        value=None,
        operator=None,
    ):
        super().__init__(
            field,
            disjunction_message_type,
            conjunction_message_type,
            message_type,
            inner_message_type,
            filters,
            value,
            operator,
        )

    def to_basic_message(self) -> Message:
        return self.message_type(
            field=self.field,
            filter_duration=self.inner_message_type(value=self.value, operator=self.operator),
        )



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from . import objects_pb2 as _objects_pb2
from . import session_status_pb2 as _session_status_pb2
from . import sessions_fields_pb2 as _sessions_fields_pb2
from . import sessions_filters_pb2 as _sessions_filters_pb2
from . import sort_direction_pb2 as _sort_direction_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SessionRaw(_message.Message):
    __slots__ = ("session_id", "status", "client_submission", "worker_submission", "partition_ids", "options", "created_at", "cancelled_at", "purged_at", "deleted_at", "duration")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    CLIENT_SUBMISSION_FIELD_NUMBER: _ClassVar[int]
    WORKER_SUBMISSION_FIELD_NUMBER: _ClassVar[int]
    PARTITION_IDS_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CANCELLED_AT_FIELD_NUMBER: _ClassVar[int]
    PURGED_AT_FIELD_NUMBER: _ClassVar[int]
    DELETED_AT_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    status: _session_status_pb2.SessionStatus
    client_submission: bool
    worker_submission: bool
    partition_ids: _containers.RepeatedScalarFieldContainer[str]
    options: _objects_pb2.TaskOptions
    created_at: _timestamp_pb2.Timestamp
    cancelled_at: _timestamp_pb2.Timestamp
    purged_at: _timestamp_pb2.Timestamp
    deleted_at: _timestamp_pb2.Timestamp
    duration: _duration_pb2.Duration
    def __init__(self, session_id: _Optional[str] = ..., status: _Optional[_Union[_session_status_pb2.SessionStatus, str]] = ..., client_submission: bool = ..., worker_submission: bool = ..., partition_ids: _Optional[_Iterable[str]] = ..., options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., cancelled_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., purged_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., deleted_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class ListSessionsRequest(_message.Message):
    __slots__ = ("page", "page_size", "filters", "sort", "with_task_options")
    class Sort(_message.Message):
        __slots__ = ("field", "direction")
        FIELD_FIELD_NUMBER: _ClassVar[int]
        DIRECTION_FIELD_NUMBER: _ClassVar[int]
        field: _sessions_fields_pb2.SessionField
        direction: _sort_direction_pb2.SortDirection
        def __init__(self, field: _Optional[_Union[_sessions_fields_pb2.SessionField, _Mapping]] = ..., direction: _Optional[_Union[_sort_direction_pb2.SortDirection, str]] = ...) -> None: ...
    PAGE_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    SORT_FIELD_NUMBER: _ClassVar[int]
    WITH_TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    page: int
    page_size: int
    filters: _sessions_filters_pb2.Filters
    sort: ListSessionsRequest.Sort
    with_task_options: bool
    def __init__(self, page: _Optional[int] = ..., page_size: _Optional[int] = ..., filters: _Optional[_Union[_sessions_filters_pb2.Filters, _Mapping]] = ..., sort: _Optional[_Union[ListSessionsRequest.Sort, _Mapping]] = ..., with_task_options: bool = ...) -> None: ...

class ListSessionsResponse(_message.Message):
    __slots__ = ("sessions", "page", "page_size", "total")
    SESSIONS_FIELD_NUMBER: _ClassVar[int]
    PAGE_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    sessions: _containers.RepeatedCompositeFieldContainer[SessionRaw]
    page: int
    page_size: int
    total: int
    def __init__(self, sessions: _Optional[_Iterable[_Union[SessionRaw, _Mapping]]] = ..., page: _Optional[int] = ..., page_size: _Optional[int] = ..., total: _Optional[int] = ...) -> None: ...

class GetSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class GetSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class CancelSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class CancelSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class CreateSessionRequest(_message.Message):
    __slots__ = ("default_task_option", "partition_ids")
    DEFAULT_TASK_OPTION_FIELD_NUMBER: _ClassVar[int]
    PARTITION_IDS_FIELD_NUMBER: _ClassVar[int]
    default_task_option: _objects_pb2.TaskOptions
    partition_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, default_task_option: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., partition_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateSessionReply(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class PauseSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class PauseSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class ResumeSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class ResumeSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class PurgeSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class PurgeSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class DeleteSessionRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class DeleteSessionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...

class StopSubmissionRequest(_message.Message):
    __slots__ = ("session_id", "client", "worker")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_FIELD_NUMBER: _ClassVar[int]
    WORKER_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    client: bool
    worker: bool
    def __init__(self, session_id: _Optional[str] = ..., client: bool = ..., worker: bool = ...) -> None: ...

class StopSubmissionResponse(_message.Message):
    __slots__ = ("session",)
    SESSION_FIELD_NUMBER: _ClassVar[int]
    session: SessionRaw
    def __init__(self, session: _Optional[_Union[SessionRaw, _Mapping]] = ...) -> None: ...



from typing import Dict

from grpc import Channel

from ..protogen.client.versions_service_pb2_grpc import VersionsStub
from ..protogen.common.versions_common_pb2 import (
    ListVersionsRequest,
    ListVersionsResponse,
)


class ArmoniKVersions:
    def __init__(self, grpc_channel: Channel):
        """Result service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = VersionsStub(grpc_channel)

    def list_versions(self) -> Dict[str, str]:
        """Get versions of ArmoniK components.

        Return:
            A dictionnary mapping each component to its version.
        """
        request = ListVersionsRequest()
        response: ListVersionsResponse = self._client.ListVersions(request)
        return {"core": response.core, "api": response.api}



from __future__ import annotations

from typing import Dict, List, Optional, Tuple, cast

from grpc import Channel

from ..common import Direction, Task, TaskDefinition, TaskOptions, TaskStatus
from ..common.filter import (
    DateFilter,
    DurationFilter,
    Filter,
    NumberFilter,
    StatusFilter,
    StringFilter,
)
from ..common.helpers import batched
from ..protogen.client.tasks_service_pb2_grpc import TasksStub
from ..protogen.common.sort_direction_pb2 import SortDirection
from ..protogen.common.tasks_common_pb2 import (
    CancelTasksRequest,
    CountTasksByStatusRequest,
    CountTasksByStatusResponse,
    GetResultIdsRequest,
    GetResultIdsResponse,
    GetTaskRequest,
    GetTaskResponse,
    ListTasksDetailedResponse,
    ListTasksRequest,
    ListTasksResponse,
    SubmitTasksRequest,
)
from ..protogen.common.tasks_fields_pb2 import (
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAME,
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE,
    TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE,
    TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION,
    TASK_OPTION_ENUM_FIELD_ENGINE_TYPE,
    TASK_OPTION_ENUM_FIELD_MAX_DURATION,
    TASK_OPTION_ENUM_FIELD_MAX_RETRIES,
    TASK_OPTION_ENUM_FIELD_PARTITION_ID,
    TASK_OPTION_ENUM_FIELD_PRIORITY,
    TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT,
    TASK_SUMMARY_ENUM_FIELD_CREATED_AT,
    TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION,
    TASK_SUMMARY_ENUM_FIELD_ENDED_AT,
    TASK_SUMMARY_ENUM_FIELD_ERROR,
    TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID,
    TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID,
    TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME,
    TASK_SUMMARY_ENUM_FIELD_POD_TTL,
    TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION,
    TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT,
    TASK_SUMMARY_ENUM_FIELD_SESSION_ID,
    TASK_SUMMARY_ENUM_FIELD_STARTED_AT,
    TASK_SUMMARY_ENUM_FIELD_STATUS,
    TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT,
    TASK_SUMMARY_ENUM_FIELD_TASK_ID,
    TaskField,
    TaskOptionField,
    TaskOptionGenericField,
    TaskSummaryField,
)
from ..protogen.common.tasks_filters_pb2 import (
    FilterField as rawFilterField,
)
from ..protogen.common.tasks_filters_pb2 import (
    Filters as rawFilters,
)
from ..protogen.common.tasks_filters_pb2 import (
    FiltersAnd as rawFilterAnd,
)
from ..protogen.common.tasks_filters_pb2 import (
    FilterStatus as rawFilterStatus,
)


class TaskFieldFilter:
    """
    Enumeration of the available filters
    """

    TASK_ID = StringFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_TASK_ID)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    SESSION_ID = StringFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_SESSION_ID)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    OWNER_POD_ID = StringFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    INITIAL_TASK_ID = StringFilter(
        TaskField(
            task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID)
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    STATUS = StatusFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_STATUS)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
        rawFilterStatus,
    )
    CREATED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_CREATED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    SUBMITTED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    STARTED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_STARTED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    ENDED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_ENDED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    CREATION_TO_END_DURATION = DurationFilter(
        TaskField(
            task_summary_field=TaskSummaryField(
                field=TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION
            )
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    PROCESSING_TO_END_DURATION = DurationFilter(
        TaskField(
            task_summary_field=TaskSummaryField(
                field=TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION
            )
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    POD_TTL = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_POD_TTL)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    POD_HOSTNAME = StringFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    RECEIVED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    ACQUIRED_AT = DateFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    ERROR = StringFilter(
        TaskField(task_summary_field=TaskSummaryField(field=TASK_SUMMARY_ENUM_FIELD_ERROR)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )

    MAX_DURATION = DurationFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_MAX_DURATION)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    MAX_RETRIES = NumberFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_MAX_RETRIES)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    PRIORITY = NumberFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_PRIORITY)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    PARTITION_ID = StringFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_PARTITION_ID)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    APPLICATION_NAME = StringFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_APPLICATION_NAME)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    APPLICATION_VERSION = StringFilter(
        TaskField(
            task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION)
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    APPLICATION_NAMESPACE = StringFilter(
        TaskField(
            task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE)
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    APPLICATION_SERVICE = StringFilter(
        TaskField(
            task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE)
        ),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )
    ENGINE_TYPE = StringFilter(
        TaskField(task_option_field=TaskOptionField(field=TASK_OPTION_ENUM_FIELD_ENGINE_TYPE)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )

    @staticmethod
    def task_options_key(option_key: str) -> StringFilter:
        """
        Filter for the TaskOptions.Options dictionary
        Args:
            option_key: key in the dictionary

        Returns:
            Corresponding filter
        """
        return StringFilter(
            TaskField(task_option_generic_field=TaskOptionGenericField(field=option_key)),
            rawFilters,
            rawFilterAnd,
            rawFilterField,
        )


class ArmoniKTasks:
    def __init__(self, grpc_channel: Channel):
        """Tasks service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = TasksStub(grpc_channel)

    def get_task(self, task_id: str) -> Task:
        """Get task information from task id

        Args:
            task_id: Id of the task

        Returns:
            Task object with the information
        """
        task_response: GetTaskResponse = self._client.GetTask(GetTaskRequest(task_id=task_id))
        return Task.from_message(task_response.task)

    def list_tasks(
        self,
        task_filter: Filter | None = None,
        with_errors: bool = False,
        page: int = 0,
        page_size: int = 1000,
        sort_field: Filter = TaskFieldFilter.TASK_ID,
        sort_direction: SortDirection = Direction.ASC,
        detailed: bool = True,
    ) -> Tuple[int, List[Task]]:
        """List tasks

        If the total returned exceeds the requested page size, you may want to use this function again and ask for subsequent pages.

        Args:
            task_filter: Filter for the tasks to be listed
            with_errors: Retrieve the error if the task had errors, defaults to false
            page: page number to request, this can be useful when paginating the result, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field on which to sort the resulting list, defaults to the task_id
            sort_direction: direction of the sort, defaults to ascending
            detailed: Wether to retrieve the detailed description of the task.

        Returns:
            A tuple containing :
            - The total number of tasks for the given filter
            - The obtained list of tasks
        """
        request = ListTasksRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, task_filter.to_disjunction().to_message())
            if task_filter
            else rawFilters(),
            sort=ListTasksRequest.Sort(
                field=cast(TaskField, sort_field.field), direction=sort_direction
            ),
            with_errors=with_errors,
        )
        if detailed:
            response: ListTasksDetailedResponse = self._client.ListTasksDetailed(request)
            return response.total, [Task.from_message(t) for t in response.tasks]
        response: ListTasksResponse = self._client.ListTasks(request)
        return response.total, [Task.from_message(t) for t in response.tasks]

    def cancel_tasks(self, task_ids: List[str], chunk_size: Optional[int] = 500):
        """Cancel tasks.

        Args:
            task_ids: IDs of the tasks.
            chunk_size: Batch size for cancelling.

        Return:
            The list of cancelled tasks.
        """
        for task_id_batch in batched(task_ids, chunk_size):
            request = CancelTasksRequest(task_ids=task_id_batch)
            self._client.CancelTasks(request)

    def get_result_ids(
        self, task_ids: List[str], chunk_size: Optional[int] = 500
    ) -> Dict[str, List[str]]:
        """Get result IDs of a list of tasks.

        Args:
            task_ids: The IDs of the tasks.
            chunk_size: Batch size for retrieval.

        Return:
            A dictionary mapping the ID of a task to the IDs of its results..
        """
        tasks_result_ids = {}

        for task_ids_batch in batched(task_ids, chunk_size):
            request = GetResultIdsRequest(task_id=task_ids_batch)
            result_ids_response: GetResultIdsResponse = self._client.GetResultIds(request)
            for t in result_ids_response.task_results:
                tasks_result_ids[t.task_id] = list(t.result_ids)
        return tasks_result_ids

    def count_tasks_by_status(self, task_filter: Filter | None = None) -> Dict[TaskStatus, int]:
        """Get number of tasks by status.

        Args:
            task_filter: Filter for the tasks to be listed

        Return:
            A dictionnary mapping each status to the number of filtered tasks.
        """
        request = CountTasksByStatusRequest(
            filters=cast(rawFilters, task_filter.to_disjunction().to_message())
            if task_filter
            else rawFilters()
        )
        count_tasks_by_status_response: CountTasksByStatusResponse = (
            self._client.CountTasksByStatus(request)
        )
        return {
            TaskStatus(status_count.status): status_count.count
            for status_count in count_tasks_by_status_response.status
        }

    def submit_tasks(
        self,
        session_id: str,
        tasks: List[TaskDefinition],
        default_task_options: Optional[TaskOptions | None] = None,
        chunk_size: Optional[int] = 100,
    ) -> List[Task]:
        """Submit tasks to ArmoniK.

        Args:
            session_id: Session Id
            tasks: List of task definitions
            default_task_options: Default Task Options used if a task has its options not set
            chunk_size: Batch size for submission

        Returns:
            Tuple containing the list of successfully sent tasks, and
            the list of submission errors if any
        """
        for tasks_batch in batched(tasks, chunk_size):
            task_creations = []

            for t in tasks_batch:
                task_creation = SubmitTasksRequest.TaskCreation(
                    expected_output_keys=t.expected_output_ids,
                    payload_id=t.payload_id,
                    data_dependencies=t.data_dependencies,
                    task_options=t.options.to_message() if t.options else None,
                )
                task_creations.append(task_creation)

            request = SubmitTasksRequest(
                session_id=session_id,
                task_creations=task_creations,
                task_options=default_task_options.to_message() if default_task_options else None,
            )

            self._client.SubmitTasks(request)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ResultRawEnumField(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    RESULT_RAW_ENUM_FIELD_UNSPECIFIED: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_SESSION_ID: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_NAME: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_OWNER_TASK_ID: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_STATUS: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_CREATED_AT: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_COMPLETED_AT: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_RESULT_ID: _ClassVar[ResultRawEnumField]
    RESULT_RAW_ENUM_FIELD_SIZE: _ClassVar[ResultRawEnumField]
RESULT_RAW_ENUM_FIELD_UNSPECIFIED: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_SESSION_ID: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_NAME: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_OWNER_TASK_ID: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_STATUS: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_CREATED_AT: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_COMPLETED_AT: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_RESULT_ID: ResultRawEnumField
RESULT_RAW_ENUM_FIELD_SIZE: ResultRawEnumField

class ResultRawField(_message.Message):
    __slots__ = ("field",)
    FIELD_FIELD_NUMBER: _ClassVar[int]
    field: ResultRawEnumField
    def __init__(self, field: _Optional[_Union[ResultRawEnumField, str]] = ...) -> None: ...

class ResultField(_message.Message):
    __slots__ = ("result_raw_field",)
    RESULT_RAW_FIELD_FIELD_NUMBER: _ClassVar[int]
    result_raw_field: ResultRawField
    def __init__(self, result_raw_field: _Optional[_Union[ResultRawField, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: applications_filters.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import applications_fields_pb2 as applications__fields__pb2
from . import filters_common_pb2 as filters__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61pplications_filters.proto\x12 armonik.api.grpc.v1.applications\x1a\x19\x61pplications_fields.proto\x1a\x14\x66ilters_common.proto\"\x9f\x01\n\x0b\x46ilterField\x12\x41\n\x05\x66ield\x18\x01 \x01(\x0b\x32\x32.armonik.api.grpc.v1.applications.ApplicationField\x12:\n\rfilter_string\x18\x02 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterStringH\x00\x42\x11\n\x0fvalue_condition\"H\n\nFiltersAnd\x12:\n\x03\x61nd\x18\x01 \x03(\x0b\x32-.armonik.api.grpc.v1.applications.FilterField\"C\n\x07\x46ilters\x12\x38\n\x02or\x18\x01 \x03(\x0b\x32,.armonik.api.grpc.v1.applications.FiltersAndB#\xaa\x02 ArmoniK.Api.gRPC.V1.Applicationsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'applications_filters_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002 ArmoniK.Api.gRPC.V1.Applications'
  _globals['_FILTERFIELD']._serialized_start=114
  _globals['_FILTERFIELD']._serialized_end=273
  _globals['_FILTERSAND']._serialized_start=275
  _globals['_FILTERSAND']._serialized_end=347
  _globals['_FILTERS']._serialized_start=349
  _globals['_FILTERS']._serialized_end=416
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ListVersionsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListVersionsResponse(_message.Message):
    __slots__ = ("core", "api")
    CORE_FIELD_NUMBER: _ClassVar[int]
    API_FIELD_NUMBER: _ClassVar[int]
    core: str
    api: str
    def __init__(self, core: _Optional[str] = ..., api: _Optional[str] = ...) -> None: ...
